/*
*  AeroLS  -- An application that computes the aerodynamic characteristics of a thin lifting surface
*
*  Copyright (C) 2002-2024, Joseph A. Huwaldt
*
*  This program is free software; you can redistribute it and/or
*  modify it under the terms of the GNU General Public License
*  as published by the Free Software Foundation; either version 2
*  of the License, or (at your option) any later version.
*  
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/gpl.html
*/

import jahuwaldt.io.ExtFilenameFilter;
import jahuwaldt.js.ParamTools.ParamException;
import jahuwaldt.swing.*;
import java.awt.Frame;
import java.awt.desktop.*;
import java.awt.event.ActionEvent;
import java.io.File;
import java.net.URL;
import java.util.List;
import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;


/**
*  An application for analyzing the aerodynamic characteristics
*  of winged vehicles using a thin lifting surface method.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author    Joseph A. Huwaldt,    Date:  May 14, 2002
*  @version   January 21, 2024
*/
public class AeroLS extends MDIApplication {

	// Some strings of information about this program.
	private static final String APP_NAME = "AeroLS Lifting Surface Aero";	//	Application/Applet Title or Name.
	private static final String	APP_VERSION = "1.2.2";						//	Version string.
	private static final String AUTHOR = "Joseph A. Huwaldt";			//	Author string.
	private static final String MOD_DATE = "January 21, 2024";			//	Modification date string.
	private static final String CW_YEAR = "2002-2024";					//	Copywrite year.

	// Definition of this application's menus.
	private static final 
		String[][] FILEM_STRINGS = {		{ "New", "N", "handleNew" },
										{ "Open...", "O", "handleOpen" },
										{ null, null, null },
										{ "Close", "W", "handleClose" },
										{ "Save", "S", null },
										{ "Save As...", null, null } };

	private static final 
		String[][] EDITM_STRINGS = { 	{ "Undo", "Z", null },
										{ null, null, null },
										{ "Cut", "X", null },
										{ "Copy", "C", null },
										{ "Paste", "V", null } };


	//	The extension used for data files.
	private static final String EXTENSION = ".alsx";

    //-----------------------------------------------------------------------------------
    /**
     * Entry point for this application. This is where it starts up from.
     */
    public static void main(String[] args) {

        // All code that creates or interacts with Swing components must run on the
        // event dispatch thread.
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                //	Display splash screen at applications startup.
                URL resource = AeroLS.class.getResource("images/SplashScreen.jpg");
                new SplashScreen(resource, null, SplashScreen.ONE_SECOND*6);
            }
        });
        
        try {
            Thread.sleep(SplashScreen.ONE_SECOND*3);
        } catch (Exception e) { }
        
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                try {
                    // Set the system look and feel to hide that hideous Java LAF.
                    AppUtilities.setSystemLAF();

                    // MacOS X users like the menu bar to be at the top of the screen.
                    // This does nothing on non-MacOS platforms.
                    System.setProperty("apple.laf.useScreenMenuBar", "true");

                    // Create an instance of our application to get things rolling.
                    new AeroLS();

                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                    AppUtilities.showException(null, "No Such Method Error",
                            "Copy the following message and e-mail it to the author:", e);
                    System.exit(0);

                } catch (ParamException e) {
                    JOptionPane.showMessageDialog(null, e.getMessage(), "Parameter Error",
                            JOptionPane.ERROR_MESSAGE);
                    e.printStackTrace();
                    System.exit(0);

                } catch (Throwable e) {
                    e.printStackTrace();
                    AppUtilities.showException(null, "Unexpected Error",
                            "Copy the following message and e-mail it to the author:", e);
                    System.exit(0);
                }
            }
        });

    }


    /**
    *  Construct a new AeroLS application and open an empty window.
    */
    @SuppressWarnings("OverridableMethodCallInConstructor")
    public AeroLS() throws ParamException, NoSuchMethodException, Exception {
        super(APP_NAME);

		// Set up the file name filter.
        ExtFilenameFilter fnFilter = new ExtFilenameFilter();
        setFilenameFilter(fnFilter);
		fnFilter.addExtension( EXTENSION );
		fnFilter.addExtension( "VL0" );
		fnFilter.addExtension( "ggp" );
		
        // Add an open files handler for MacOS.
        setOpenFileHandler(new OpenFilesHandler() {
            @Override
            public void openFiles(OpenFilesEvent e) {
                try {
                    //  Loop over the files to be opened and open each in turn.
                    List<File> files = e.getFiles();
                    for (File file : files) {
                        AppWindow.newWindowFromDataFiles(file);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        });
        
		//  Create the menu bar.
		JMenuBar menuBar = createMenuBar();
	    
		// Add the menu bar to this application.
		//  Cause the application to stay open when the last window is closed under MacOS.
		//  When the last window is closed, a menu bar will remain for the application with no
		//  window open.  This is standard MacOS behavior and will not happen on other platforms.
		setFramelessJMenuBar( menuBar );
		
		//  Create a main window frame for this application.
		AppWindow window = new AppWindow();
		addWindow(window);
		window.setVisible(true);
    }
    

	/**
	*  Create an about handler for use in this application.
	*/
    private AboutHandler createAboutHandler() {

        AboutHandler about = new AboutHandler() {
            @Override
            public void handleAbout(AboutEvent e) {
                
				//  Load in the application's icon image.
				Icon appIcon = null;
				try {
					URL imgURL = ClassLoader.getSystemResource("images/Icon64x64.gif");
					appIcon = new ImageIcon(imgURL);
				} catch (Exception ex) {
                    ex.printStackTrace();
                }
				
                if (appIcon == null)
                    //  Use a generic icon since this app can't read it's custom icon.
                    appIcon = UIManager.getIcon("OptionPane.informationIcon");
                
				//  Create a standard looking dialog box that displays the credits.
				String credits = "<html><body><b>Author:</b><br>" + AUTHOR + "<br>";
				credits += "<a href=\"mailto:jhuwaldt@mac.com\">jhuwaldt@mac.com</a><br>";
				credits += "<br><b>Special thanks to:</b>";
				credits += "<br><b>JScience</b><br><a href=\"http://jscience.org/\">http://jscience.org/</a><br>";
				credits += "by Jean-Marie Dautelle: &nbsp;<a href=\"mailto:jean-marie@dautelle.com\">jean-marie@dautelle.com</a><br>";
				credits += "<P ALIGN=CENTER><BR>" + APP_NAME + " comes with ABSOLUTELY NO WARRANTY;";
				credits += "<BR>This is free software, and you are welcome to redistribute ";
				credits += "it and it's source code under certain conditions.";
				credits += "<BR>Source code is available at:";
				credits += "<BR><a href=\"http://thehuwaldtfamily.org/java/Applications/AeroLS/AeroLS.html\">";
				credits += "http://thehuwaldtfamily.org/java/Applications/AeroLS/AeroLS.html</a></P>";
				credits += "</body></html>";
				StandardMacAboutFrame f = new StandardMacAboutFrame(APP_NAME, APP_VERSION + " - " + MOD_DATE);
				f.setApplicationIcon(appIcon);
				f.setCopyright("Copyright " + CW_YEAR + ", " + AUTHOR);
				f.setCredits(credits, "text/html");
                f.setHyperlinkListener(new HyperlinkListener() {
                    @Override
                    public void hyperlinkUpdate(HyperlinkEvent e) {
                        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                            //  Display the selected URL in the default browser for this platform.
                            try {
                                MDIApplication.browse(e.getURL().toURI());
                            } catch (Exception err) {
                                err.printStackTrace();
                            }
                        }
                    }
                });
                f.setVisible(true);
			}
		};
		
		return about;
	}


	/**
	*  Handles the user choosing "Open" from the File menu.
	*  Allows the user to choose an file to open.
	*/
	public void handleOpen( ActionEvent event ) {
        //  Create a new document window.
        AppWindow window = (AppWindow) handleNew(null);

        //  Have the new window handle opening the files.
        window.handleOpen(null);
	}
	
	/**
	*  Handle the user requesting a new document window.
	*  Creates a new instance of this application's main document window.
	*
	*  @param  event  The event that caused this method to be called.  May be "null"
	*                 if this method is called by MDIApplication or one of it's subclasses.
	*  @return A reference to the window created or null if no window is created.
	*/
    @Override
	public Frame handleNew( ActionEvent event) {
		AppWindow window = null;
		
		try {
		
			window = new AppWindow();
			window.setVisible(true);
			addWindow(window);
			
        } catch (NoSuchMethodException e) {
			e.printStackTrace();
			AppUtilities.showException(null, "No Such Method Error",
						"Copy the following message and e-mail it to the author:", e);
		
		} catch (ParamException e) {
		    JOptionPane.showMessageDialog( null, e.getMessage(), "Parameter Error",
								    JOptionPane.ERROR_MESSAGE );
            e.printStackTrace();
            
		} catch (Throwable e) {
			e.printStackTrace();
			AppUtilities.showException(null, "Unexpected Error",
							"Copy the following message and e-mail it to the author:", e);
		}
		
		return window;
	}
	
	/**
	*  Initializes the menus associated with this window.
	*/
	private JMenuBar createMenuBar() throws NoSuchMethodException {

		JMenuBar menuBar = new JMenuBar();
		
		// Set up the file menu.
		JMenu menu = AppUtilities.buildMenu( this, "File", FILEM_STRINGS );
		menuBar.add( menu );

        //  Add a Quit menu item (if it isn't automatically present).
        if (!QuitJMenuItem.isAutomaticallyPresent()) {
            menu.addSeparator();
            menu.add(this.getQuitJMenuItem());
        }
        
        //  Add a Quit handler to ask the user if they want to save unsaved changes before quitting.
        setQuitHandler(new QuitHandler() {
            @Override
            public void handleQuitRequestWith(QuitEvent e, QuitResponse response) {
                handleQuit(e, response);
            }
        });
        
		// Set up the edit menu.
		menu = AppUtilities.buildMenu( this, "Edit", EDITM_STRINGS );
		menuBar.add( menu );

        //	Register an about menu item handler.
        setAboutHandler(createAboutHandler());
        
        //  Create an about menu item (if not automatically present).
        if (!AboutJMenuItem.isAutomaticallyPresent()) {
            //  Create Help menu.
            menu = new JMenu("Help");
            menuBar.add(menu);
            
            //  Add the "About" item to the Help menu.
            menu.add(this.getAboutJMenuItem());
        }
        
		return menuBar;
	}

}

