/*
 *  AnalysisThread  -- Thread that carries out the analysis done by the AeroLS program.
 *
 *   Copyright (C) 2002-2024 by Joseph A. Huwaldt
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.aero.AeroReference;
import jahuwaldt.js.ParamTools.Param;
import jahuwaldt.js.tables.FTableList;
import jahuwaldt.swing.AppUtilities;
import jahuwaldt.swing.MDIApplication;
import jahuwaldt.swing.ProgressBarHandler;
import java.awt.Component;
import javax.measure.quantity.Area;
import javax.measure.quantity.Length;
import javax.measure.unit.SI;
import javax.measure.unit.Unit;
import javax.swing.JOptionPane;

/**
 * A thread that analyzes the specified geometry at the specified test
 * conditions.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: June 13, 2002
 * @version January 21, 2024
 */
public class AnalysisThread extends Thread implements ParamNames {

    //  Debug flag.
    private static final boolean DEBUG = false;

    //  Reference to this project's data container.
    private final ProjectData pdata;

    //	The parent component of this layer array.
    private final Component parentComp;

    /**
     * Construct this analysis thread.
     *
     * @param data A reference to the data structure for this model.
     * @param parent This is the parent component used for displaying error
     * dialogs.
     */
    public AnalysisThread(ProjectData data, Component parent) {
        pdata = data;
        parentComp = parent;
    }

    /**
     * Method that actually does the analysis. This method is called
     * automatically when the user executes the Thread.start() method. Do not
     * call this method directly!
     */
    @Override
    public void run() {
        
        if (DEBUG)
            System.out.println("Starting analysis...");

        //  Check to make sure there is geometry to analyze.
        if (pdata.geometry.getNumberOfPanels() < 1) {
            JOptionPane.showMessageDialog(null, "There is no geometry to analyze.", "No Geometry",
                    JOptionPane.ERROR_MESSAGE);
            return;
        }

        //	Show a progress dialog.
        ProgressBarHandler progBar = new ProgressBarHandler("Analyzing \"" + pdata.modelName + "\"...", parentComp);
        
        try {

            //	Extract the reference geometry.
            Param param = pdata.pdb.get(SREF);
            double Sref = param.getValue();
            double SrefRU = param.get().doubleValue(Area.UNIT);
            Unit<Area> areaUnit = param.getUnit();
            param = pdata.pdb.get(CREF);
            double crefRU = param.get().doubleValue(Length.UNIT);
            Unit<Length> lengthUnit = param.getUnit();
            param = pdata.pdb.get(BREF);
            double brefRU = param.get().doubleValue(Length.UNIT);
            param = pdata.pdb.get(XMRC);
            double XmrcRU = param.get().doubleValue(Length.UNIT);
            param = pdata.pdb.get(YMRC);
            double YmrcRU = param.get().doubleValue(Length.UNIT);
            param = pdata.pdb.get(ZMRC);
            double ZmrcRU = param.get().doubleValue(Length.UNIT);
            AeroReference ref = new AeroReference(Sref, areaUnit, crefRU, brefRU,
                    XmrcRU, YmrcRU, ZmrcRU, SI.METER);
            ref.changeLengthUnits(lengthUnit);
            if (DEBUG)
                System.out.println("Sref = " + SrefRU + ", cref = " + crefRU + ", bref = " + brefRU);

            //  Extract the Mach number breakpoints.
            double[] machBP = pdata.testConditions.get(pdata.indepNames[0]);

            //  Extract the angle of attack breakpoints.
            double[] alphaBP = pdata.testConditions.get(pdata.indepNames[1]);

            //  Extract the sideslip angle in degrees.
            double beta = pdata.beta;

            //	Generate the geometry related data that we need.
            VLGeometry geom = new VLGeometry(pdata.geometry, 0.75, true);
            progBar.setProgress(0.01F);

            //  Inputs required to calculate the downwash matrix.
            double wakeTE = pdata.geometry.getMaxY() * 100;
            double tol = crefRU / 20;

            //  Create an analysis object.
            VLAnalysis analysis = new VLAnalysis(geom, ref, machBP, alphaBP, beta, tol, wakeTE);

            //  Run the analysis.
            FTableList tables = analysis.run(progBar, 0.99F);
            if (DEBUG)
                System.out.println("tables = " + tables);
            
            progBar.setProgress(1);

            //  Display the results.
            TableWindow resultsWindow = new TableWindow(pdata.modelName + " Results", tables);
            resultsWindow.pack();

            //  Position the window so that each new window can be seen.
            AppUtilities.positionWindow(resultsWindow, resultsWindow.getWidth(), resultsWindow.getHeight());
            
            resultsWindow.setVisible(true);
            MDIApplication.addWindow(resultsWindow);
            
        } catch (InterruptedException e) {
			//	If the user cancels, just return.
            
        } catch (Throwable e) {
            System.err.println("Error:  " + e);
            e.printStackTrace();
            JOptionPane.showMessageDialog(parentComp, e.toString(), "Unknown Error",
                    JOptionPane.ERROR_MESSAGE);
        }
        
        progBar.setProgress(1);
        
        if (DEBUG)
            System.out.println("Analysis complete.");
    }
    
}
