/*
 *  FrontViewPanel  -- A panel that draws a front view of the geometry in AeroLS.
 *
 *   Copyright (C) 2002-2015 by Joseph A. Huwaldt
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.aero.panelgeom.GeomNetwork;
import jahuwaldt.aero.panelgeom.GeomVehicle;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;

/**
 * A panel component that draws a front view of the specified geometry.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: May 29, 2002
 * @version June 20, 2015
 */
public class FrontViewPanel extends ViewPanel {

    //  The transformation to apply to the geometry before it is rendered.
    private AffineTransform at = AffineTransform.getScaleInstance(scale, -scale);

    //-------------------------------------------------------------------------
    /**
     * Creates a view panel that displays the front view of the given geometry.
     *
     * @param geom The geometry to be displayed.
     */
    public FrontViewPanel(GeomVehicle geom) {
        super(geom);
    }

    /**
     * This method sets the model drawing scale.
     *
     * @param denom The denominator of the scale factor (e.g.: 100 for 1/100
     * scale).
     */
    @Override
    public void setScale(int denom) {
        super.setScale(denom);

        at = AffineTransform.getScaleInstance(scale, -scale);
    }

    //-------------------------------------------------------------------------
    /**
     * Method that returns an appropriate transformation for this view panel.
     * Typically, this involves a translation to center the model in the panel.
     */
    @Override
    protected AffineTransform getTransform(Dimension panelSize) {

        //  Get the bounds of our geometry.
        double minY = vehicle.getMinY() * scale;
        double maxY = vehicle.getMaxY() * scale;
        double minZ = vehicle.getMinZ() * scale;
        double maxZ = vehicle.getMaxZ() * scale;

        //  Create a translation that will center the model in this panel.
        AffineTransform transform = new AffineTransform();
        transform.translate(panelSize.width / 2 + (maxY + minY) / 2, panelSize.height / 2 + (maxZ + minZ) / 2);
        transform.rotate(-Math.PI / 2);
        transform.concatenate(at);

        return transform;
    }

    /**
     * Method that renders a wire frame model of a single network with a "front
     * view" orientation.
     */
    @Override
    protected void renderWFNetwork(Graphics2D gc, GeomNetwork network) {

        //  Draw the panels first.
        gc.setStroke(thinLine);

        double[][] zArr = network.getAllZ();
        double[][] yArr = network.getAllY();

        int numCols = yArr.length - 1;      //  Spanwise.
        int numRows = yArr[0].length - 1;   //  Chordwise.
        for (int i = 0; i < numCols; ++i) {
            double[] zArri = zArr[i];
            double[] yArri = yArr[i];
            double[] zArrip1 = zArr[i + 1];
            double[] yArrip1 = yArr[i + 1];

            for (int j = 0; j < numRows; ++j) {
                GeneralPath polygon = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 5);

                int jp1 = j + 1;
                polygon.moveTo(zArri[j], yArri[j]);
                polygon.lineTo(zArri[jp1], yArri[jp1]);
                polygon.lineTo(zArrip1[jp1], yArrip1[jp1]);
                polygon.lineTo(zArrip1[j], yArrip1[j]);
                polygon.closePath();

                gc.draw(polygon);
            }
        }

        //  Now draw the outline of the network itself.
        gc.setStroke(thickLine);
        GeneralPath polygon = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 5);

        polygon.moveTo(zArr[0][0], yArr[0][0]);
        polygon.lineTo(zArr[0][numRows], yArr[0][numRows]);
        polygon.lineTo(zArr[numCols][numRows], yArr[numCols][numRows]);
        polygon.lineTo(zArr[numCols][0], yArr[numCols][0]);
        polygon.closePath();

        gc.draw(polygon);

    }

    /**
     * Method that returns a line representing the plane of symmetry for this
     * view.
     */
    @Override
    protected Line2D createSymmetryLine() {
        double vlength = vehicle.getMaxZ() - vehicle.getMinZ();
        Line2D line = new Line2D.Double(-0.25 * vlength, 0, 1.25 * vlength, 0);
        return line;
    }

    /**
     * Method that returns the name of this view. This method returns "Front
     * View".
     */
    @Override
    protected String getViewName() {
        return "Front View";
    }

}
