/*
 *  GGPReader  -- A class that enables reading of GGP geometry files in AeroLS.
 *
 *   Copyright (C) 2002-2015 by Joseph A. Huwaldt
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.aero.panelgeom.GGPGeomReader;
import jahuwaldt.aero.panelgeom.GeomConfig;
import jahuwaldt.aero.panelgeom.GeomElementList;
import jahuwaldt.aero.panelgeom.GeomReader;
import jahuwaldt.js.ParamTools.ParamDatabase;
import java.io.IOException;
import java.io.InputStream;
import javax.measure.quantity.Length;
import javax.measure.unit.Unit;
import javax.swing.JOptionPane;

/**
 * A class that reads in geometry data from a GGP formatted ASCII file. GGP is a
 * geometry file format often used in conjunction with solution output from A502
 * (PANAIR) and A633 (TRANAIR). This implementation will only read in parameters
 * from the GGP file with the names "X", "Y", and "Z". All other parameters are
 * ignored. This implementation also assumes that all the parameters are
 * contained in a single line and that the 1st format is duplicated throughout
 * the file.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: July 10, 2002
 * @version June 23, 2015
 */
public class GGPReader implements ParamNames {

    /**
     * Method that reads in a GGP formatted text stream and creates a new model
     * from it.
     *
     * @param inStream The input stream to VL0 DAT file that we are reading from.
     * @param pdb The parameter database containing the parameters for this project.
     * @return A new ProjectData object with the data from the file filled in.
     */
    public static ProjectData read(InputStream inStream, ParamDatabase pdb) throws IOException, Exception {

        //  Create a new project data container.
        ProjectData pdata = new ProjectData(pdb);

        //  Ask the user to select the units used in the file.
        Unit<Length> units = askForUnits();

	    //  Create a GGP geometry reader object.
        //	Make use of a GGP reader that is part of the jahuwaldt.aero.panelgeom package.
        GeomReader reader = new GGPGeomReader();
        reader.setFileUnits(units);

        //  Read in and parse the geometry.
        GeomConfig config = reader.read(inStream, "GGP Geometry");

        //  GGPGeomReader always returns a configuration with a single vehicle
        //  with a single component.  Pull out the vehicle object.
        pdata.geometry.add(((GeomElementList) config.get(0)).get(0));

        return pdata;
    }

    /**
     * Ask the user to select from a list of unit types.
     */
    private static Unit<Length> askForUnits() throws InterruptedException {

        //  Extract the names of the length units from the list used by this program.
        int numUnits = LENGTH_UNITS.length;
        String[] names = new String[numUnits];
        for (int i = 0; i < numUnits; ++i)
            names[i] = LENGTH_UNITS[i].toString();

        //  Ask the user to choose one.
        String selectedValue = (String) JOptionPane.showInputDialog(null,
                "Choose the units used in the file", "Input Units",
                JOptionPane.INFORMATION_MESSAGE, null,
                names, names[0]);

        //  Check for user canceling.
        if (selectedValue == null)
            throw new InterruptedException("User canceled.");

        //  Figure out which one the user chose.
        int idx = 0;
        for (int i = 0; i < numUnits; ++i)
            if (selectedValue.equals(names[i])) {
                idx = i;
                break;
            }

        return LENGTH_UNITS[idx];
    }

}
