/*
*  LookupWindow  -- A window that enables interactive table lookups on the specified analysis table.
*
*   Copyright (C) 2000-2024 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
*/

import jahuwaldt.js.tables.Float64Table;
import jahuwaldt.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javolution.text.Text;
import org.jscience.mathematics.vector.Float64Vector;


/**
*  A window that displays a set of sliders and text fields for
*  doing table lookups on a selected table.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author    Joseph A. Huwaldt    Date:  February 24, 2000
*  @version   January 21, 2024
*/
public class LookupWindow extends JFrame {

	// Definition of this window's menus.
	private static final 
		String[][] FILEM_STRINGS ={		{ "New", null, null },
										{ "Open...", null, null },
										{ null, null, null },
										{ "Close", "W", "handleClose" },
										{ "Save", null, null } };
		
	private static final 
		String[][] EDITM_STRINGS = {		{ "Undo", "Z", null },
										{ null, null, null },
										{ "Cut", "X", null },
										{ "Copy", "C", null },
										{ "Paste", "V", null } };

	// The resolution of the slider scale.
	private static final int SLIDER_SCALE = (Integer.MAX_VALUE) / 100;

	// The tick mark increment for the slider scale.
	private static final int TICK_SPACING = SLIDER_SCALE / 10;

	// A copy of the table we are doing a lookup for.
	private final Float64Table table;

	// The minimum and maximum breakpoints for the table.
	private final double[] minBP;

	private final double[] maxBP;

	// The independent variables we want to do a look up for.
	private final double[] indep;

	// A copy of the sliders used in this window, one for each dimension.
	private final JSlider[] sliders;

	// A copy of the text fields used by this window, one for each dimension.
	private final DecimalField[] textFields;

	// The text field that holds the result of the lookup.
	private final DecimalField result;


	/**
	*  Creates a new instance of a table lookup window using the
	*  specified table.
	*
	*  @param  app       A reference to the main application class.
	*  @param  theTable  The table we want to do a table lookup on.
	*/
    @SuppressWarnings("OverridableMethodCallInConstructor")
	public LookupWindow( Float64Table theTable ) throws NoSuchMethodException {
		super( "Lookup for " + theTable.getName().toString() );

        //  Have the window dispose of itself when it closes.
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

		this.setResizable( true );
			
		// Set up some class variables.
		this.table = theTable;
		int numDims = theTable.getDimensions();
		minBP = new double [ numDims ];
		maxBP = new double [ numDims ];
		indep = new double [ numDims ];
		sliders = new JSlider [ numDims ];
		textFields = new DecimalField [ numDims ];
		
		// Create some event listeners.
		SliderListener sliderL = new SliderListener();
		TextFieldListener textL = new TextFieldListener();
		TextFieldFocusL focusL = new TextFieldFocusL();
		
		// Define the format we are going to use for the text fields.
		DecimalFormat format = (DecimalFormat) NumberFormat.getNumberInstance();
		format.setMaximumFractionDigits( 5 );
		format.setMinimumFractionDigits( 1 );
		
		// Set up this window's user interface.
		JPanel grid1 = new JPanel( new GridLayout( 1, numDims, 10, 10 ) );
		grid1.setBorder( BorderFactory.createEmptyBorder( 10, 10, 10, 10 ) );
		grid1.setAlignmentX( JPanel.CENTER_ALIGNMENT );
		
		// Create a label, slider, and text edit box for each independent variable.
		List<Text> indepNames = theTable.getIndepNames();
		for ( int i = 0; i < numDims; ++i ) {
			
			// Create a slider for this independent variable.
			Float64Vector bpoints = theTable.getBreakpoints( i );
			minBP[i] = bpoints.getValue(0);
			maxBP[i] = bpoints.getValue(bpoints.getDimension()- 1);
			indep[i] = minBP[i];
			JSlider slider = new JSlider( JSlider.VERTICAL, 0, SLIDER_SCALE, 0 );
			slider.addChangeListener( sliderL );
			slider.setMajorTickSpacing( TICK_SPACING );
			slider.setPaintTicks( true );
			Integer dimension = i;
			slider.putClientProperty( "dimension", dimension );
			
			// Create labels for the top and bottom of the slider.
			Hashtable labelTable = new Hashtable();
			labelTable.put(0, new JLabel( Double.toString( minBP[i] ) ) );
			labelTable.put(SLIDER_SCALE, new JLabel( Double.toString( maxBP[i] ) ) );
			slider.setLabelTable( labelTable );
			slider.setPaintLabels( true );
			slider.setBorder( BorderFactory.createEmptyBorder( 0, 10, 5, 0 ) );
            if (bpoints.getDimension() == 1)
                slider.setEnabled(false);
			sliders[i] = slider;
			
			// Create a text area for this slider.
			DecimalField textField = new DecimalField( minBP[i], 7, format );
			textField.addActionListener( textL );
			textField.addFocusListener( focusL );
			textField.putClientProperty( "dimension", dimension );
            if (bpoints.getDimension() == 1)
                textField.setEnabled(false);
			textFields[i] = textField;
			
			// Layout the independent panel.
			JPanel aPanel = new JPanel( new BorderLayout() );
			TitledBorder tborder = BorderFactory.createTitledBorder( indepNames.get(i).toString() );
			tborder.setTitleJustification( TitledBorder.CENTER );
			aPanel.setBorder( tborder );
			aPanel.add( slider, BorderLayout.CENTER );
			aPanel.add( textField, BorderLayout.SOUTH );
			
			// Add the independent panel to the window's grid layout.
			grid1.add( aPanel );
		}
		
        // Make it so you can tab from on text field to another.
        List<Component> order = new ArrayList();
        for (int i = 0; i < numDims; ++i) {
            order.add(textFields[i]);
        }
        this.setFocusTraversalPolicy(new OrderedFocusTraversalPolicy(order));
        
		// Create a table label.
		JLabel tableL = new JLabel( theTable.getName() + " = " );
		
		// Create a text field that contains the look up value.
		result = new DecimalField( 0, 10, format );
		result.setEditable( false );
		doLookup();
		
		JPanel botPanel = new JPanel( new FlowLayout() );
		botPanel.add( tableL );
		botPanel.add( result );
		
		// Add everything to the window's pane.
		getContentPane().add( grid1, BorderLayout.CENTER );
		getContentPane().add( botPanel, BorderLayout.SOUTH );

		//  Create a menu bar for this window.
		JMenuBar menuBar = createMenuBar();
		this.setJMenuBar(menuBar);

        //  Size the window.
        this.pack();

        //  Position the window so that each new window can be seen.
        AppUtilities.positionWindow(this, getWidth(), getHeight());
    }

    /**
     * A FocusTraversalPolicy that cycles through a supplied list of Component
     * objects.
     */
    public static class OrderedFocusTraversalPolicy extends FocusTraversalPolicy {
        private final List<Component> order;
        
        public OrderedFocusTraversalPolicy(List<Component> order) {
            this.order = new ArrayList(order.size());
            this.order.addAll(order);
        }

        @Override
        public Component getComponentAfter(Container focusCycleRoot, Component aComponent) {
            int idx = (order.indexOf(aComponent) + 1) % order.size();
            return order.get(idx);
        }
        
        @Override
        public Component getComponentBefore(Container focusCycleRoot, Component aComponent) {
            int idx = order.indexOf(aComponent) - 1;
            if (idx < 0) {
                idx = order.size() - 1;
            }
            return order.get(idx);
        }
        
        @Override
        public Component getDefaultComponent(Container focusCycleRoot) {
            return order.get(0);
        }
        
        @Override
        public Component getLastComponent(Container focusCycleRoot) {
            return order.get(order.size() - 1);
        }
        
        @Override
        public Component getFirstComponent(Container focusCycleRoot) {
            return order.get(0);
        }
    }

	/**
	*  Initializes the menus associated with this window.
	*/
	private JMenuBar createMenuBar() throws NoSuchMethodException {

		JMenuBar menuBar = new JMenuBar();
		
		// Set up the file menu.
		JMenu menu = AppUtilities.buildMenu( this, "File", FILEM_STRINGS );
		menuBar.add( menu );

        //  Add a Quit menu item (if it isn't automatically present).
        AeroLS guiApp = (AeroLS)MDIApplication.getInstance();
        if (!QuitJMenuItem.isAutomaticallyPresent()) {
            menu.addSeparator();
            menu.add(guiApp.getQuitJMenuItem());
        }

		// Set up the edit menu.
		menu = AppUtilities.buildMenu( this, "Edit", EDITM_STRINGS );
		menuBar.add( menu );

		//  Create a Window's menu.
		menu = MDIApplication.newWindowsMenu("Windows");
		menuBar.add(menu);
		
        //  Create an about menu item (if not automatically present).
        if (!AboutJMenuItem.isAutomaticallyPresent()) {
            //  Create Help menu.
            menu = new JMenu("Help");
            menuBar.add(menu);
            
            //  Add the "About" item to the Help menu.
            menu.add(guiApp.getAboutJMenuItem());
        }
        
		return menuBar;
	}


	/**
	*  This method actually does the table lookup and stores the
	*  result in the result text field.
	*/
	private void doLookup() {
		double value = table.lookupValue( indep );
		result.setValue( value );
	}

	/**
	*  Handles the user changing the value in a text field.
	*
	*  @param  input  The text field that has had it's value changed.
	*/
	private void doTextFieldChange( DecimalField input ) {
		
		// Find out what dimension this is and what value it has been set to.
		int i = ((Integer) input.getClientProperty("dimension"));
        double value = (double) input.getValue();
		
		// Validate the user's input.
		if ( value > maxBP[i] )
			value = maxBP[i];
		
		else {
			if ( value < minBP[i] )
				value = minBP[i];
		}
		
		// Update the slider to match the text input.
		int val = (int) ((value - minBP[i])/(maxBP[i] - minBP[i])*SLIDER_SCALE);
		sliders[i].setValue( val );
		
		// Do the table lookup.
		indep[i] = value;
		doLookup();
	}


	/**
	*  Handle the user choosing "Close" from the File menu.  This implementation
	*  dispatches a "Window Closing" event to this window.
	*/
	public void handleClose( ActionEvent event ) {
		this.dispatchEvent( new WindowEvent( this, WindowEvent.WINDOW_CLOSING ) );
	}


	/**
	*  Defines a listener for our sliders that is called whenever the slider
	*  is changed.
	*/
	private class SliderListener extends Object implements ChangeListener {

        @Override
		public void stateChanged( ChangeEvent e ) {
			JSlider input = (JSlider) e.getSource();
			
			// Find out what dimension this is and what value it has been set to.
			int i = ((Integer) input.getClientProperty( "dimension" ));
			int val = input.getValue();
			
			// Update the text field to match the slider.
			double value = (maxBP[i] - minBP[i])*val/SLIDER_SCALE + minBP[i];
			textFields[i].setValue( value );
			
			// Do the table lookup.
			indep[i] = value;
			doLookup();
		}
	}
	
	/**
	*  Defines a listener for our text fields that is called whenever the user
	*  types in a new number and presses enter.
	*/
	private class TextFieldListener extends Object implements ActionListener {

        @Override
		public void actionPerformed( ActionEvent e ) {
			DecimalField input = (DecimalField) e.getSource();
			doTextFieldChange( input );
		}


	}
	/**
	*  Defines a listener for our text fields that is called whenever the
	*  focus changes off a particular text field.
	*/
	private class TextFieldFocusL extends Object implements FocusListener {

        @Override
		public void focusGained( FocusEvent e ) { }

        @Override
		public void focusLost( FocusEvent e ) {
			DecimalField input = (DecimalField) e.getSource();
			doTextFieldChange( input );
		}
	}

}


