/*
 *  NewWingSegDialog  -- A dialog for defining a trapazoidal wing segment in AeroLS.
 *
 *   Copyright (C) 2002-2024 by Joseph A. Huwaldt
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.aero.panelgeom.GeomComponent;
import jahuwaldt.aero.panelgeom.GeomNetwork;
import jahuwaldt.js.ParamTools.*;
import jahuwaldt.swing.AppUtilities;
import jahuwaldt.swing.EscapeJDialog;
import java.awt.*;
import java.awt.event.*;
import java.net.MalformedURLException;
import javax.measure.unit.SI;
import javax.measure.unit.Unit;
import javax.swing.*;

/**
 * A dialog the allows the user to enter/change the values of the various
 * properties that can be used to define a trapezoidal wing segment or network.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: May 22, 2002
 * @version January 21, 2024
 */
public class NewWingSegDialog extends EscapeJDialog implements ParamNames {

    //  A reference to the main application window.
    private final AppWindow mainWindow;

    //  The component we are adding this segment to.
    private final GeomComponent compRef;

    //  A text field to contain the model name.
    private final JTextField nameTField;

    //  The number of parameters used in this dialog.
    private static final int NUM_GEOM_PARAMS = 20;
    
    //  A parameter database of only those parameters used by this dialog.
    private ParamDatabase pdb = new ParamDatabase();
    
    //  The chordwise distribution option chosen by the user.
    private int rowDistSelection = 0;

    //  The spanwise distribution option chosen by the user.
    private int colDistSelection = 0;
    
    //  Panel distribution options.
    //private static final int LINEAR = 0;
    private static final int ONE_M_COS = 1;
    private static final int SINE = 2;
    private static final int ACOS = 3;

    
    /**
    *  Construct the dialog.
    *
    *  @param  parent  A reference to the AeroLS main window that this dialog
    *                  belongs to.
    *  @param  title   The title for the dialog window.
    *  @param  comp    The component we are adding this segment to.
    *  @param  pdb     The parameter database containing the
    *                  properties used to define a trapezoidal wing segment.
    */
    @SuppressWarnings("OverridableMethodCallInConstructor")
    public NewWingSegDialog(AppWindow parent, String title, GeomComponent comp, ParamDatabase params)
                        throws MalformedURLException {
        super(parent, title);

        mainWindow = parent;
        compRef = comp;

        //  Layout the dialog window.
		Container cp = this.getContentPane();
		cp.setLayout(new BorderLayout());

		//  Create a panel to contain our inputs.
		JPanel inputs = new JPanel(new BorderLayout());
		cp.add(inputs, BorderLayout.CENTER);

        //  Create an input box for the segment name.
		JPanel namePanel = new JPanel(new BorderLayout());
		namePanel.setBorder(BorderFactory.createEmptyBorder(6,0,6,12));
		Box box = Box.createHorizontalBox();
		namePanel.add(box, BorderLayout.WEST);
		nameTField = new JTextField( "Untitled", 20 );
		JLabel label = new JLabel("Segment Name = ", JLabel.RIGHT);
		label.setLabelFor(nameTField);
		box.add(label);
		box.add(nameTField);
		box.add(Box.createGlue());
		inputs.add(namePanel, BorderLayout.NORTH);
		
        //  Display input boxes for all the geometry parameters in this window.
		String[] pLabels = new String[NUM_GEOM_PARAMS];
		int i = 0;
		pLabels[i] = XRLE;	        ++i;
		pLabels[i] = YRLE;	        ++i;
		pLabels[i] = ZRLE;		    ++i;
		pLabels[i] = XTLE;	        ++i;
		pLabels[i] = YTLE;	        ++i;
		pLabels[i] = ZTLE;		    ++i;
		pLabels[i] = BSEG;	    	++i;
		pLabels[i] = ARSEG;		++i;
		pLabels[i] = SSEG;	    	++i;
		pLabels[i] = CRSEG;		++i;
		pLabels[i] = CTSEG;		++i;
		pLabels[i] = TAPER_SEG;	    ++i;
		pLabels[i] = SWEEPLE_SEG;	++i;
		pLabels[i] = SWEEPC4_SEG;	++i;
		pLabels[i] = SWEEPTE_SEG;	++i;
		pLabels[i] = MAC_SEG;	    ++i;
		pLabels[i] = GAMSEG;       ++i;
		pLabels[i] = TWIST;        ++i;
		pLabels[i] = I_ROOT;        ++i;
		pLabels[i] = I_TIP;         ++i;

		//  Add the specified parameters to our local parameter database.
		for (i=0; i < NUM_GEOM_PARAMS; ++i)
		    pdb.put(pLabels[i], params.get(pLabels[i]));
		    
		JPanel paramPanel = layoutOrderedParams(pdb, 3, pLabels);
		paramPanel.setBorder(BorderFactory.createTitledBorder("Segment Geometry"));
		inputs.add(paramPanel, BorderLayout.CENTER);


        //  Display input boxes for panel distribution parameters.
		paramPanel = layoutPanelDistributionPanel(params);
		paramPanel.setBorder(BorderFactory.createTitledBorder("Panel Distribution"));
		inputs.add(paramPanel, BorderLayout.SOUTH);

        
        //  Put some instructions at the top.
		box = Box.createHorizontalBox();
		cp.add(box, BorderLayout.NORTH);
		JLabel instructions = new JLabel("Define a new wing segment by entering known geometry:");
		instructions.setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
		box.add(instructions);
		box.add(Box.createGlue());

		//  Create a clear all button and put it at the top too.
		JButton clearAllButton = new JButton("Clear All");
		box.add(clearAllButton);
		box.add(Box.createHorizontalStrut(10));
		clearAllButton.setToolTipText("Resets all the parameters in this window.");
		clearAllButton.addActionListener(new ActionListener() {
            @Override
            public  void actionPerformed( ActionEvent evt ) {
			    AutoParam.clearAllParams(pdb);
            }
		});


        //  Define OK and Cancel buttons.
		box = Box.createHorizontalBox();
		box.add(Box.createGlue());

        JButton cancelBtn = new JButton("Cancel");
		cancelBtn.addActionListener( new ActionListener() {
            @Override
		    public void actionPerformed(ActionEvent e) {
                NewWingSegDialog.this.performEscapeAction(null);
		    }
		});
		box.add(cancelBtn);
		
		box.add(Box.createHorizontalStrut(40));
        
        JButton okayBtn = new JButton("Create");
		okayBtn.addActionListener( new ActionListener() {
            @Override
		    public void actionPerformed(ActionEvent e) {
		        createSegmentAndClose();
		    }
		});
		this.getRootPane().setDefaultButton(okayBtn);
		box.add(okayBtn);
		
		box.add(Box.createHorizontalStrut(20));
		cp.add(box, BorderLayout.SOUTH);

		
		// Define a window listener that handles closing the window.
		this.addWindowListener( new WindowAdapter() {
            @Override
			public  void windowClosing( WindowEvent e ) {
				NewWingSegDialog.this.performEscapeAction(null);
			}
		} );
		
    }


	/**
	*  Defines the panel that allows the user to specify the panel distribution options.
	*
	*  @param  params   The overall parameter database being used by this program.
	*  @param  pLabels  An list of string labels for the parameters that are
	*                   to be displayed.
	*  @return A panel containing all the parameter input boxes.
	*/
	private JPanel layoutPanelDistributionPanel(ParamDatabase params)
	                                throws MalformedURLException {
		String[] pLabels = new String[2];
		pLabels[0] = ROWS;
		pLabels[1] = COLUMNS;
		
		//  Add the specified parameters to our local parameter database.
		for (int i=0; i < 2; ++i)
		    pdb.put(pLabels[i], params.get(pLabels[i]));
		    
        JPanel panel = new JPanel(new GridLayout(1,5));

        //  1st add the row parameter.
		Param theParam = pdb.get(pLabels[0]);
		ParamInput thePI = new ParamInput( theParam );
        panel.add(thePI);

        //  Create a distrubtion menu for the rows.
        String[] distOptions = {"Linear", "1 - Cosine", "Sine", "Arc Cosine"};
        JComboBox comboBox = new JComboBox(distOptions);
        comboBox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JComboBox cb = (JComboBox)e.getSource();
                rowDistSelection = cb.getSelectedIndex();
                
            }
        });
        panel.add(comboBox);
        panel.add(new JLabel(" "));
        
        //  2nd add the column parameter.
		theParam = pdb.get(pLabels[1]);
		thePI = new ParamInput( theParam );
        panel.add(thePI);

        //  Create a distrubtion menu for the rows.
        comboBox = new JComboBox(distOptions);
        comboBox.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                JComboBox cb = (JComboBox)e.getSource();
                colDistSelection = cb.getSelectedIndex();
                
            }
        });
        panel.add(comboBox);
        
        return panel;
	}
	

    /**
     * Layout the selected parameters in the parameter database by adding them
     * as components in a grid bag layout to a new panel and returning that panel.
     *
     * @param pdb The parameter database being used by this program.
     * @param columns The number of columns to use when displaying parameters.
     * @param pLabels An list of string labels for the parameters that are to be displayed.
     * @return A panel containing all the parameter input boxes.
     */
    private static JPanel layoutOrderedParams(ParamDatabase pdb, int columns, String[] pLabels)
            throws MalformedURLException {
        int i = 0;
        int xPos = 0, yPos = 0;

        //	Set up for grid layout.
        GridBagConstraints constraints = new GridBagConstraints();
        GridBagLayout gridbag = new GridBagLayout();
        JPanel panel = new JPanel(gridbag);

        //	Loop over elements in database, placing each one in this container.
        int length = pLabels.length;
        while (i < length) {
            AppUtilities.buildConstraints(constraints, xPos, yPos, 1, 1, 100 / columns, 10);
            constraints.fill = GridBagConstraints.BOTH;		//	How the component should fill cell.
            constraints.anchor = GridBagConstraints.CENTER;	//	Justification in cell

            //	Get the next element from database.
            AdvParam theParam = (AdvParam) pdb.get(pLabels[i]);

            if (theParam == null)
                throw new NullPointerException("Parameter " + pLabels[i] + " not found in database.");

            //	Create text field for inputing this parameter's value.
            AdvParamInput thePI = new AdvParamInput(theParam);
            gridbag.setConstraints(thePI, constraints);
            panel.add(thePI);

            ++xPos;
            if (xPos >= columns) {
                xPos = 0;
                ++yPos;
            }

            ++i;
        }

        return panel;
    }

    /**
    *  Extract the information the user has entered and create a new
    *  wing segment form it.  Then update the main window and
    *  close this one.
    */
	private void createSegmentAndClose() {

        if (inputsAvailable(pdb)) {

            //  Extract the name of the segment.
            String name = nameTField.getText();
            if (name == null || name.equals(""))
                name = "Untitled";

            //  Create a new geometry network (segment).
            GeomNetwork network = createSegment(pdb, name, rowDistSelection, colDistSelection);

            //  Add the new network to the component it belongs to.
            compRef.add(network);

            //  Update the display of the main window.
            mainWindow.update();

            //  Close the dialog window and dispose of it.
            NewWingSegDialog.this.setVisible(false);
            NewWingSegDialog.this.dispose();
            
        } else {
		    JOptionPane.showMessageDialog( this, "Not enough information has been entered to\n" +
		                                    "create a new wing segment.", "Insufficient Information",
								    JOptionPane.WARNING_MESSAGE );
        }
        
	}

    /**
    *  Checks to see if the required inputs have been entered
    *  by the user or not.
    */
    private static boolean inputsAvailable(ParamDatabase pdb) {
        boolean result = true;

        String[] reqParams = {  XRLE, YRLE, ZRLE,
                                XTLE, YTLE, ZTLE,
                                CRSEG, CTSEG, ROWS,
                                COLUMNS, I_ROOT, I_TIP };

        int length = reqParams.length;
        for (int i=0; i < length; ++i) {
            Param param = pdb.get(reqParams[i]);
            if (!param.isDefined()) {
                result = false;
                break;
            }
        }
        
        return result;
    }
    
    /**
    *  Method that creates a new wing segment using the user
    *  input geometry information.
    */
    private static GeomNetwork createSegment(ParamDatabase pdb, String name,
                                                int rowDistOption, int colDistOption) {

        //  Extract the required geometry information.
        //  All geometry is extracted in meters.
        double x0 = pdb.get(XRLE).get().doubleValue((Unit)SI.METER);
        double y0 = pdb.get(YRLE).get().doubleValue((Unit)SI.METER);
        double z0 = pdb.get(ZRLE).get().doubleValue((Unit)SI.METER);
        double x1 = pdb.get(XTLE).get().doubleValue((Unit)SI.METER);
        double y1 = pdb.get(YTLE).get().doubleValue((Unit)SI.METER);
        double z1 = pdb.get(ZTLE).get().doubleValue((Unit)SI.METER);
        double cr = pdb.get(CRSEG).get().doubleValue((Unit)SI.METER);
        double ct = pdb.get(CTSEG).get().doubleValue((Unit)SI.METER);
        double ir = pdb.get(I_ROOT).get().doubleValue((Unit)SI.RADIAN);
        double it = pdb.get(I_TIP).get().doubleValue((Unit)SI.RADIAN);

        int rows = (int)pdb.get(ROWS).getValue() + 1;
        int cols = (int)pdb.get(COLUMNS).getValue() + 1;

        //  Generate the distribution of points.
        double[] rowDist = createDistribution(rows, rowDistOption);
        double[] colDist = createDistribution(cols, colDistOption);

        //  Allocate memory for arrays.
        double[][] xpnts = new double[cols][rows];
        double[][] ypnts = new double[cols][rows];
        double[][] zpnts = new double[cols][rows];

        //  Generate the points from inboard to outboard.
        for (int i=0; i < cols; ++i) {
            double dist = colDist[i];
            double y = y0 + dist*(y1 - y0);
            double chord = cr + dist*(ct - cr);
            double xle = x0 + dist*(x1 - x0);
            double zle = z0 + dist*(z1 - z0);
            double incidence = ir + dist*(it - ir);
            double dz = -chord*Math.tan(incidence);
            
			//	Generate points from leading edge to trailing edge.
            for (int j=0; j < rows; ++j) {
                dist = rowDist[j];
                double x = xle + dist*chord;
                double z = zle + dist*dz;

                xpnts[i][j] = x;
                ypnts[i][j] = y;
                zpnts[i][j] = z;
            }
        }

        //  Create the network object.
        GeomNetwork net = new GeomNetwork(xpnts, ypnts, zpnts, name);
        
        return net;
    }


    /**
    *  Method that generates a distribution of points in the range
    *  0.0 to 1.0 inclusive.
    */
    private static double[] createDistribution(int numPoints, int distOption) {
        double[] dist = new double[numPoints];
        
        switch (distOption) {
            case ONE_M_COS:
                {
                    double inc = Math.PI/2/(numPoints-1);
                    double angle = 0;
                    for (int i=0; i < numPoints; ++i) {
                        double d = (1 - Math.cos(angle));
                        if (d < 0) d = 0;
                        else if (d > 1) d = 1;
                        dist[i] = d;
                        angle += inc;
                    }       break;
                }
            case SINE:
                {
                    double inc = Math.PI/2/(numPoints-1);
                    double angle = 0;
                    for (int i=0; i < numPoints; ++i) {
                        double d = Math.sin(angle);
                        if (d < 0) d = 0;
                        else if (d > 1) d = 1;
                        dist[i] = d;
                        angle += inc;
                    }       break;
                }
            case ACOS:
                for (int i=0; i < numPoints; ++i) {
                    double angle = Math.acos((double)i/(double)(numPoints - 1));
                    double d = 1 - angle/Math.PI*2;
                    if (d < 0) d = 0;
                    else if (d > 1) d = 1;
                    dist[i] = d;
                }   break;
            default:
                //  Use a linear distribution.
                for (int i=0; i < numPoints; ++i) {
                        dist[i] = (double)i/(double)(numPoints-1);
                        }   break;
        }

        return dist;
    }

    
	/**
	*  Response to ENTER key pressed goes here.
	*  This method does nothing.
	*/
    @Override
	protected void performEnterAction(KeyEvent e) { }

	/**
	*  Response to ESCAPE key pressed goes here.
	*  This is the same as clicking on the Cancel button,
	*  or closing the dialog with the window close box.
	*/
    @Override
	protected void performEscapeAction(KeyEvent e) {
	    
        //  Close dialog window without creating a new wing segment.
	    this.setVisible(false);
	    this.dispose();
	}

}

