/*
 *  ProjectData  -- A class for containing project data for the AeroLS program.
 *  Copyright (C) 2002-2015, Joseph A. Huwaldt
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/gpl.html
 */

import jahuwaldt.aero.panelgeom.GeomVehicle;
import jahuwaldt.js.ParamTools.AdvParam;
import jahuwaldt.js.ParamTools.AutoParam;
import jahuwaldt.js.ParamTools.ParamDatabase;
import jahuwaldt.js.ParamTools.ParamException;
import jahuwaldt.js.ParamTools.ParamMethod;
import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.pmethods.winggeom.*;
import jahuwaldt.util.GeneralFormat;
import java.util.Hashtable;
import javax.measure.quantity.Dimensionless;
import javax.measure.unit.NonSI;

/**
 * A simple class that serves as a container for project data.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: May 31, 2002
 * @version June 23, 2015
 */
public class ProjectData implements ParamNames {

    /**
     * The name of the currently loaded model.
     */
    String modelName = "Untitled";

    /**
     * The parameter database containing this application's parameters.
     */
    ParamDatabase pdb = new ParamDatabase();

    /**
     * The geometry of the currently loaded model.
     */
    final GeomVehicle geometry = new GeomVehicle(modelName);

    /**
     * The test condition sweeps to be analyzed. Should include an array of Mach
     * number breakpoints and an array of angle of attack in degrees
     * breakpoints.
     */
    final Hashtable<String,double[]> testConditions = new Hashtable();
    final String[] indepNames = {"Mach", "AOA"};

    /**
     * Angle of sideslip in degrees.
     */
    double beta = 0;

    /**
     * Construct a ProjectData object with the default settings.
     */
    public ProjectData() throws ParamException {

        //  Define all the parameters in the parameter database used by this program.
        defineParamDatabase();

        //  Create a default set of test conditions.
        //  Mach number breakpoints.
        double[] breakPoints = new double[5];
        for (int i = 0; i < 5; ++i)
            breakPoints[i] = i * 0.2;
        testConditions.put(indepNames[0], breakPoints);

        //  Angle of attack breakpoints.
        breakPoints = new double[11];
        double value = -4;
        for (int i = 0; i < 11; ++i, value += 2)
            breakPoints[i] = value;
        testConditions.put(indepNames[1], breakPoints);

    }

    /**
     * Construct a ProjectData object using an existing parameter database.
     */
    public ProjectData(ParamDatabase pdb) {
        this.pdb = pdb;

        //  Create a default set of test conditions.
        //  Mach number breakpoints.
        double[] breakPoints = new double[5];
        for (int i = 0; i < 5; ++i)
            breakPoints[i] = i * 0.2;
        testConditions.put("Mach", breakPoints);

        //  Angle of attack breakpoints.
        breakPoints = new double[11];
        double value = -4;
        for (int i = 0; i < 11; ++i, value += 2)
            breakPoints[i] = value;
        testConditions.put("AOA", breakPoints);

    }

    /**
     * Method that defines all the parameters used by this program and places
     * them in a parameter database with unique (and hopefully descriptive)
     * names.
     */
    private void defineParamDatabase() throws ParamException {

        try {
            //  Define the format of the numbers displayed to the user.
            GeneralFormat nf = new GeneralFormat();

            //  Define the aero reference quantities.
            //	Sref parameter
            AdvParam aParam1 = new AdvParam(SREF, 1, NonSI.SQUARE_FOOT);
            aParam1.setAvailUnits(AREA_UNITS);
            aParam1.setNumberFormat(nf);
            pdb.put(SREF, aParam1);

            //	bref parameter
            aParam1 = new AdvParam(BREF, 1, NonSI.FOOT);
            aParam1.setAvailUnits(LENGTH_UNITS);
            aParam1.setNumberFormat(nf);
            pdb.put(BREF, aParam1);

            //	cref parameter
            aParam1 = new AdvParam(CREF, 1, NonSI.FOOT);
            aParam1.setAvailUnits(LENGTH_UNITS);
            aParam1.setNumberFormat(nf);
            pdb.put(CREF, aParam1);

            //	MRC X coordinate parameter
            aParam1 = new AdvParam(XMRC, 0, NonSI.FOOT);
            aParam1.setAvailUnits(LENGTH_UNITS);
            aParam1.setNumberFormat(nf);
            pdb.put(XMRC, aParam1);

            //	MRC Y coordinate parameter
            aParam1 = new AdvParam(YMRC, 0, NonSI.FOOT);
            aParam1.setAvailUnits(LENGTH_UNITS);
            aParam1.setNumberFormat(nf);
            pdb.put(YMRC, aParam1);

            //	MRC Z coordinate parameter
            aParam1 = new AdvParam(ZMRC, 0, NonSI.FOOT);
            aParam1.setAvailUnits(LENGTH_UNITS);
            aParam1.setNumberFormat(nf);
            pdb.put(ZMRC, aParam1);

            //	bseg (segment span) parameter
            ParamMethod method = new bfn_S_AR(SSEG, ARSEG);
            AutoParam aParam = new AutoParam(BSEG, NonSI.FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setParamVerifier(new bVerifier());
            method = new bfn_S_Cr_Ct(SSEG, CRSEG, CTSEG);
            aParam.addMethod(method);
            method = new bfn_AR_Cr_Ct(ARSEG, CRSEG, CTSEG);
            aParam.addMethod(method);
            method = new bfn_yrle_ytle(YRLE, YTLE);
            aParam.addMethod(method);
            aParam.setAvailUnits(LENGTH_UNITS);

            //	ARseg parameter
            method = new ARfn_b_S(BSEG, SSEG);
            aParam = new AutoParam(ARSEG, Dimensionless.UNIT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setParamVerifier(new ARVerifier());
            method = new ARfn_b_Cr_Ct(BSEG, CRSEG, CTSEG);
            aParam.addMethod(method);
            method = new ARvfn_sweepLE_sweep4_taper(SWEEPLE_SEG, SWEEPC4_SEG, TAPER_SEG);
            aParam.addMethod(method);

            //	Sseg parameter
            method = new Sfn_b_AR(BSEG, ARSEG);
            aParam = new AutoParam(SSEG, null, NonSI.SQUARE_FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setParamVerifier(new SVerifier());
            method = new Sfn_b_Cr_Ct(BSEG, CRSEG, CTSEG);
            aParam.addMethod(method);
            aParam.setAvailUnits(AREA_UNITS);

            //	Crseg parameter
            method = new Crfn_S_b_Ct(SSEG, BSEG, CTSEG);
            aParam = new AutoParam(CRSEG, null, NonSI.FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setParamVerifier(new ChordVerifier());
            method = new Crfn_taper_Ct(TAPER_SEG, CTSEG);
            aParam.addMethod(method);
            method = new Crfn_S_b_taper(SSEG, BSEG, TAPER_SEG);
            aParam.addMethod(method);
            method = new Crfn_Cbar_taper(MAC_SEG, TAPER_SEG);
            aParam.addMethod(method);
            aParam.setAvailUnits(LENGTH_UNITS);

            //	Ctseg parameter
            method = new Ctfn_S_b_Cr(SSEG, BSEG, CRSEG);
            aParam = new AutoParam(CTSEG, null, NonSI.FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setParamVerifier(new ChordVerifier());
            method = new Ctfn_taper_Cr(TAPER_SEG, CRSEG);
            aParam.addMethod(method);
            aParam.setAvailUnits(LENGTH_UNITS);

            //	Taper ratio parameter
            method = new taperfn_Cr_Ct(CRSEG, CTSEG);
            aParam = new AutoParam(TAPER_SEG, Dimensionless.UNIT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setParamVerifier(new taperVerifier());
            method = new taperfn_Cr_Cbar(CRSEG, MAC_SEG);
            aParam.addMethod(method);
            method = new taperfn_swpLE_swp4_AR(SWEEPLE_SEG, SWEEPC4_SEG, ARSEG);
            aParam.addMethod(method);

            //	MACseg parameter
            method = new Cbarfn_Cr_taper(CRSEG, TAPER_SEG);
            aParam = new AutoParam(MAC_SEG, null, NonSI.FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setParamVerifier(new ChordVerifier());
            aParam.setAvailUnits(LENGTH_UNITS);

            //	SweepLEseg parameter
            method = new sweepLEvfn_sweep4_AR_taper(SWEEPC4_SEG, ARSEG, TAPER_SEG);
            aParam = new AutoParam(SWEEPLE_SEG, null, NonSI.DEGREE_ANGLE, method, pdb);
            aParam.setNumberFormat(nf);
            method = new sweepLEvfn_sweepTE_AR_taper(SWEEPTE_SEG, ARSEG, TAPER_SEG);
            aParam.addMethod(method);
            method = new sweepLEfn_xrle_xtle_b(XRLE, XTLE, BSEG);
            aParam.addMethod(method);
            aParam.setAvailUnits(ANGULAR_UNITS);

            //	Sweep1/4cseg parameter
            method = new sweep4vfn_sweepLE_AR_taper(SWEEPLE_SEG, ARSEG, TAPER_SEG);
            aParam = new AutoParam(SWEEPC4_SEG, null, NonSI.DEGREE_ANGLE, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(ANGULAR_UNITS);

            //	SweepTEseg parameter
            method = new sweepTEvfn_sweepLE_AR_taper(SWEEPLE_SEG, ARSEG, TAPER_SEG);
            aParam = new AutoParam(SWEEPTE_SEG, null, NonSI.DEGREE_ANGLE, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(ANGULAR_UNITS);

            //	xrle parameter
            method = new xrlefn_sweepLE_xtle_b(SWEEPLE_SEG, XTLE, BSEG);
            aParam = new AutoParam(XRLE, null, NonSI.FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(LENGTH_UNITS);

            //	yrle parameter
            method = new yrlefn_b_dihed_ytle(BSEG, GAMSEG, YTLE);
            aParam = new AutoParam(YRLE, null, NonSI.FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(LENGTH_UNITS);

            //	zrle parameter
            method = new zrlefn_dihed_b_ztle(GAMSEG, BSEG, ZTLE);
            aParam = new AutoParam(ZRLE, null, NonSI.FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(LENGTH_UNITS);

            //	xtle parameter
            method = new xtlefn_sweepLE_xrle_b(SWEEPLE_SEG, XRLE, BSEG);
            aParam = new AutoParam(XTLE, null, NonSI.FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(LENGTH_UNITS);

            //	ytle parameter
            method = new ytlefn_b_dihed_yrle(BSEG, GAMSEG, YRLE);
            aParam = new AutoParam(YTLE, null, NonSI.FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(LENGTH_UNITS);

            //	ztle parameter
            method = new ztlefn_dihed_b_zrle(GAMSEG, BSEG, ZRLE);
            aParam = new AutoParam(ZTLE, null, NonSI.FOOT, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(LENGTH_UNITS);

            //	Gamseg (dihedral) parameter
            method = new dihedfn_b_zrle_ztle(BSEG, ZRLE, ZTLE);
            aParam = new AutoParam(GAMSEG, null, NonSI.DEGREE_ANGLE, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(ANGULAR_UNITS);

            //	Root incidence parameter
            method = new irfn_it_twist(I_TIP, TWIST);
            aParam = new AutoParam(I_ROOT, null, NonSI.DEGREE_ANGLE, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(ANGULAR_UNITS);

            //	Tip incidence parameter
            method = new itfn_ir_twist(I_ROOT, TWIST);
            aParam = new AutoParam(I_TIP, null, NonSI.DEGREE_ANGLE, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(ANGULAR_UNITS);

            //	Twist parameter
            method = new twistfn_ir_it(I_ROOT, I_TIP);
            aParam = new AutoParam(TWIST, null, NonSI.DEGREE_ANGLE, method, pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(ANGULAR_UNITS);

            //	Chordwise Panels (rows) parameter
            method = null;
            aParam = new AutoParam(ROWS, Dimensionless.UNIT, method, pdb);
            aParam.setValue(4);

            //	Spanwise Panels (columns) parameter
            method = null;
            aParam = new AutoParam(COLUMNS, Dimensionless.UNIT, method, pdb);
            aParam.setValue(4);

            //	Location of panel control point parameter
            aParam1 = new AdvParam(CP, Parameter.valueOf(0.75, Dimensionless.UNIT));
            pdb.put(CP, aParam1);

            //	Have each auto-parameter link up with the parameters that it depends on.
            AutoParam.linkAllParams(pdb);

        } catch (Exception e) {
            //  Shouldn't be possible if properly debugged!
            e.printStackTrace();
            System.exit(0);
        }

    }

}
