/*
 *  ReferencesDialog  -- A dialog that allows the user to input references in AeroLS.
 *
 *   Copyright (C) 2002-2024 by Joseph A. Huwaldt
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.js.ParamTools.AdvParam;
import jahuwaldt.js.ParamTools.AdvParamInput;
import jahuwaldt.js.ParamTools.ParamDatabase;
import jahuwaldt.swing.AppUtilities;
import jahuwaldt.swing.EscapeJDialog;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.*;
import java.net.MalformedURLException;
import javax.swing.*;

/**
 * A dialog the allows the user to enter/change the values of the various
 * aerodynamic reference quantities.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: May 16, 2002
 * @version January 21, 2024
 */
public class ReferencesDialog extends EscapeJDialog implements ParamNames, ActionListener {

    //  A reference to the main application window.
    private final AppWindow mainWindow;

    //  A text field to contain the model name.
    private final JTextField nameTField;

    /**
     * Construct a dialog that allows the user to enter/change the values of
     * various reference quantities.
     *
     * @param parent A reference to the AeroLS main window that this dialog
     * belongs to.
     * @param title The title for the dialog window.
     * @param pdb The parameter database containing the aerodynamic reference
     * quantities.
     */
    @SuppressWarnings("OverridableMethodCallInConstructor")
    public ReferencesDialog(AppWindow parent, String title, ParamDatabase pdb)
            throws MalformedURLException {
        super(parent, title);

        mainWindow = parent;

        //  Layout the dialog window.
        Container cp = this.getContentPane();
        cp.setLayout(new BorderLayout());

        //  Create a panel to contain our inputs.
        JPanel inputs = new JPanel(new BorderLayout());
        cp.add(inputs, BorderLayout.CENTER);

        //  Create an input box for the model name.
        JPanel namePanel = new JPanel(new BorderLayout());
        namePanel.setBorder(BorderFactory.createEmptyBorder(6, 0, 6, 12));
        nameTField = new JTextField(parent.getModelName(), 20);
        namePanel.add(nameTField, BorderLayout.CENTER);
        JLabel label = new JLabel("Model Name = ", JLabel.RIGHT);
        label.setLabelFor(nameTField);
        namePanel.add(label, BorderLayout.WEST);
        inputs.add(namePanel, BorderLayout.NORTH);

        //  Display input boxes for all the parameters in this window.
        String pLabels[] = new String[6];
        int i = 0;
        pLabels[i] = SREF; ++i;
        pLabels[i] = BREF; ++i;
        pLabels[i] = CREF; ++i;
        pLabels[i] = XMRC; ++i;
        pLabels[i] = YMRC; ++i;
        pLabels[i] = ZMRC; ++i;
        JPanel paramPanel = layoutOrderedParams(pdb, 2, pLabels);
        inputs.add(paramPanel, BorderLayout.CENTER);

        JLabel instructions = new JLabel("Enter the units and values for references:");
        instructions.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));
        cp.add(instructions, BorderLayout.NORTH);

        JButton okayBtn = new JButton("Done");
        okayBtn.addActionListener(this);
        this.getRootPane().setDefaultButton(okayBtn);

        Box box = Box.createHorizontalBox();
        box.add(Box.createGlue());
        box.add(okayBtn);
        box.add(Box.createHorizontalStrut(10));
        cp.add(box, BorderLayout.SOUTH);

        // Define a window listener that handles closing the window.
        this.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                ReferencesDialog.this.handleWindowClosing(e);
            }

        });

    }

    /**
     * Layout the selected parameters in the parameter database by adding them
     * as components in a grid bag layout to a new panel and returning that panel.
     *
     * @param pdb The parameter database being used by this program.
     * @param columns The number of columns to use when displaying parameters.
     * @param pLabels An list of string labels for the parameters that are to be displayed.
     * @return A panel containing all the parameter input boxes.
     */
    private static JPanel layoutOrderedParams(ParamDatabase pdb, int columns, String[] pLabels)
            throws MalformedURLException {
        int i = 0;
        int xPos = 0, yPos = 0;

        //	Set up for grid layout.
        GridBagConstraints constraints = new GridBagConstraints();
        GridBagLayout gridbag = new GridBagLayout();
        JPanel panel = new JPanel(gridbag);

        //	Loop over elements in database, placing each one in this container.
        int length = pLabels.length;
        while (i < length) {
            AppUtilities.buildConstraints(constraints, xPos, yPos, 1, 1, 100 / columns, 10);
            constraints.fill = GridBagConstraints.BOTH;		//	How the component should fill cell.
            constraints.anchor = GridBagConstraints.CENTER;	//	Justification in cell

            //	Get the next element from database.
            AdvParam theParam = (AdvParam) pdb.get(pLabels[i]);

            if (theParam == null)
                throw new NullPointerException("Parameter " + pLabels[i] + " not found in database.");

            //	Create text field for inputing this parameter's value.
            AdvParamInput thePI = new AdvParamInput(theParam);
            gridbag.setConstraints(thePI, constraints);
            panel.add(thePI);

            ++xPos;
            if (xPos >= columns) {
                xPos = 0;
                ++yPos;
            }

            ++i;
        }

        return panel;
    }

    /**
     * This method is called when the user clicks on the "okayBtn".
     */
    @Override
    public void actionPerformed(ActionEvent evt) {
        //  Update the display of the main window.
        mainWindow.setModelName(nameTField.getText());
        mainWindow.update();

        //  Close the window and dispose of it.
        this.setVisible(false);
        this.dispose();
    }

    /**
     * Response to ENTER key pressed goes here. This has the same result as
     * clicking on the OK or Done button.
     */
    @Override
    protected void performEnterAction(KeyEvent e) {
        actionPerformed(null);
    }

    /**
     * Response to ESCAPE key pressed goes here. This method does nothing.
     */
    @Override
    protected void performEscapeAction(KeyEvent e) { }

    /**
     * Handles the user clicking on the "close" box.
     */
    public synchronized void handleWindowClosing(WindowEvent e) {
        actionPerformed(null);
    }

}
