/*
 *  SideViewPanel  -- A panel that draws a side view of the geometry in AeroLS.
 *
 *   Copyright (C) 2002-2015 by Joseph A. Huwaldt
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.aero.panelgeom.GeomNetwork;
import jahuwaldt.aero.panelgeom.GeomVehicle;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;

/**
 * A panel component that draws a side view of the specified geometry.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: June 2, 2002
 * @version June 20, 2015
 */
public class SideViewPanel extends ViewPanel {

    //  The transformation to apply to the geometry before it is rendered.
    private AffineTransform at = AffineTransform.getScaleInstance(scale, -scale);

    //-------------------------------------------------------------------------
    /**
     * Creates a view panel that displays the side view of the given geometry.
     *
     * @param geom The geometry to be displayed.
     */
    public SideViewPanel(GeomVehicle geom) {
        super(geom);
    }

    /**
     * This method sets the model drawing scale.
     *
     * @param denom The denominator of the scale factor (e.g.: 100 for 1/100
     * scale).
     */
    @Override
    public void setScale(int denom) {
        super.setScale(denom);

        at = AffineTransform.getScaleInstance(scale, -scale);
    }

    //-------------------------------------------------------------------------
    /**
     * Method that returns an appropriate transformation for this view panel.
     * Typically, this involves a translation to center the model in the panel.
     */
    @Override
    protected AffineTransform getTransform(Dimension panelSize) {

        //  Get the bounds of our geometry.
        double minX = vehicle.getMinX() * scale;
        double maxX = vehicle.getMaxX() * scale;
        double minZ = vehicle.getMinZ() * scale;
        double maxZ = vehicle.getMaxZ() * scale;

        //  Create a translation that will center the model in this panel.
        AffineTransform transform = new AffineTransform();
        transform.translate(panelSize.width / 2 - (maxX + minX) / 2, panelSize.height / 2 + (maxZ + minZ) / 2);
        transform.concatenate(at);

        return transform;
    }

    /**
     * Method that renders a wire frame model of a single network with a "side
     * view" orientation.
     */
    @Override
    protected void renderWFNetwork(Graphics2D gc, GeomNetwork network) {

        //  Draw the panels first.
        gc.setStroke(thinLine);

        double[][] xArr = network.getAllX();
        double[][] zArr = network.getAllZ();

        int numCols = xArr.length - 1;      //  Spanwise.
        int numRows = xArr[0].length - 1;   //  Chordwise.
        for (int i = 0; i < numCols; ++i) {
            double[] xArri = xArr[i];
            double[] zArri = zArr[i];
            double[] xArrip1 = xArr[i + 1];
            double[] zArrip1 = zArr[i + 1];

            for (int j = 0; j < numRows; ++j) {
                GeneralPath polygon = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 5);

                int jp1 = j + 1;
                polygon.moveTo(xArri[j], zArri[j]);
                polygon.lineTo(xArri[jp1], zArri[jp1]);
                polygon.lineTo(xArrip1[jp1], zArrip1[jp1]);
                polygon.lineTo(xArrip1[j], zArrip1[j]);
                polygon.closePath();

                gc.draw(polygon);
            }
        }

        //  Now draw the outline of the network itself.
        gc.setStroke(thickLine);
        GeneralPath polygon = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 5);

        polygon.moveTo(xArr[0][0], zArr[0][0]);
        polygon.lineTo(xArr[0][numRows], zArr[0][numRows]);
        polygon.lineTo(xArr[numCols][numRows], zArr[numCols][numRows]);
        polygon.lineTo(xArr[numCols][0], zArr[numCols][0]);
        polygon.closePath();

        gc.draw(polygon);

    }

    /**
     * Method that returns a line representing the plane of symmetry for this
     * view. The side view has no plane of symmetry line, so zero length line is
     * returned.
     */
    @Override
    protected Line2D createSymmetryLine() {
        Line2D line = new Line2D.Float(0, 0, 0, 0);
        return line;
    }

    /**
     * Method that returns the name of this view. This method returns "Side
     * View".
     */
    @Override
    protected String getViewName() {
        return "Side View";
    }

}
