/*
*  TableViewWindow  -- A window that displays the contents of an analysis table.
*
*   Copyright (C) 2002-2024 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
*/

import jahuwaldt.io.ExtFilenameFilter;
import jahuwaldt.js.tables.CSVReader;
import jahuwaldt.js.tables.FTableList;
import jahuwaldt.js.tables.FTableReader;
import jahuwaldt.js.tables.Float64Table;
import jahuwaldt.swing.*;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.List;
import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javolution.util.FastTable;
import org.jscience.mathematics.number.Integer64;


/**
*  A window for displaying and optionally saving an analysis table.
*  Derived from "TableWindow.java" in my Plot Digitizer program.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author  Joseph A. Huwaldt,   Date:  June 27, 2002
*  @version January 21, 2024
*/
public class TableViewWindow extends JFrame {

	/**
	*  Definitions for our map window's menus.
	**/
	private static final 
		String[][] FILEM_STRINGS ={		{ "New", "N", null },
										{ "Open...", "O", null },
										{ null, null, null },
										{ "Close", "W", "handleClose" },
										{ "Save", "S", null },
										{ "Save To CSV...", null, "handleSaveAs" } };
	

	private static final 
		String[][] EDITM_STRINGS = {		{ "Undo", "Z", null },
										{ null, null, null },
										{ "Cut", "X", null },
										{ "Copy", "C", "handleCopy" },
										{ "Paste", "V", "handlePaste" } };

    private static final FTableReader CSV_READER = new CSVReader();

	//	The extension used for data files.
	private static final String EXTENSION = ".csv";

	//	The last file opened by the user.
	private File lastFile;

	//  An adapter that will place our table data on the clipboard in an
	//  Excel compatible format.
	TableClipboardAdapter clipboard;
	
	//	Number formats used to parse/format text strings.
	NumberFormat nfd = (DecimalFormat)NumberFormat.getInstance();
	NumberFormat nfi = (DecimalFormat)NumberFormat.getInstance();

	//	The table to be displayed in this window.
	Float64Table table;
	
	//	Names of the independent and dependent variables in the table.
	private String[] names;
	
	//	Array used to set position in a multi-dimensional table.
	private List<Integer64> pos;
	

	/**
	*  Constructs a table view window.
	*/
    @SuppressWarnings("OverridableMethodCallInConstructor")
	public TableViewWindow(Float64Table tbl, File lastFile) throws NoSuchMethodException {
		super( tbl.toString() );
		
		this.lastFile = lastFile;
		this.setResizable( true );

        nfi.setMinimumFractionDigits(2);
		nfd.setMinimumFractionDigits(4);
        
		//	Save off the table.
		this.table = tbl;

		//  Allocate memory for some arrays.
		int numDims = tbl.getDimensions();
		names = new String[numDims+1];
        pos = FastTable.newInstance();
        for (int i = 0; i < numDims; ++i)
            pos.add(Integer64.ZERO);
		
		//	Create a column for each independent variable followed by the dependent.
		for (int i=0; i < numDims; ++i)
		    names[i] = tbl.getIndepName(i).toString();
		names[numDims] = tbl.getName().toString();
		
        // Create a table to show the data.
        final DecimalField decField = new DecimalField(8, nfd);
        DefaultCellEditor decEditor = new DefaultCellEditor(decField) {
            //Override DefaultCellEditor's getCellEditorValue method
            //to return an Float, not a String:
            @Override
            public Object getCellEditorValue() {
                return nfd.format(decField.getValue());
            }
        };
        TableModel dataModel = new ViewTableModel();
        JTable tableView = new JTable(dataModel);
        tableView.setDefaultEditor(String.class, decEditor);
        tableView.setCellSelectionEnabled(true);
        clipboard = new TableClipboardAdapter(tableView);
        JScrollPane scrollpane = new JScrollPane(tableView);
        scrollpane.setPreferredSize(new Dimension(250, 300));

		//	Layout the main window.
		Container cp = getContentPane();
		cp.add(scrollpane, BorderLayout.CENTER);
		
		//  Create a menu bar for this window.
		JMenuBar menuBar = createMenuBar();
		this.setJMenuBar(menuBar);
	}
	

	/**
	*  Initializes the menus associated with this window.
	*/
	private JMenuBar createMenuBar() throws NoSuchMethodException {

		JMenuBar menuBar = new JMenuBar();
		
		// Set up the file menu.
		JMenu menu = AppUtilities.buildMenu( this, "File", FILEM_STRINGS );
		menuBar.add( menu );

        //  Add a Quit menu item (if it isn't automatically present).
        MDIApplication guiApp = MDIApplication.getInstance();
        if (!QuitJMenuItem.isAutomaticallyPresent()) {
            menu.addSeparator();
            menu.add(guiApp.getQuitJMenuItem());
        }

		// Set up the edit menu.
		menu = AppUtilities.buildMenu( this, "Edit", EDITM_STRINGS );
		menuBar.add( menu );

		//  Create a Window's menu.
		menu = MDIApplication.newWindowsMenu("Windows");
		menuBar.add(menu);
		
        //  Create an about menu item (if not automatically present).
        if (!AboutJMenuItem.isAutomaticallyPresent()) {
            //  Create Help menu.
            menu = new JMenu("Help");
            menuBar.add(menu);
            
            //  Add the "About" item to the Help menu.
            menu.add(guiApp.getAboutJMenuItem());
        }
		
		return menuBar;
	}


	/**
	*  Handles the user choosing "Copy" from the Edit menu.
	*/
	public void handleCopy( ActionEvent event ) {
	    clipboard.copy();
	}
	
	/**
	*  Handles the user choosing "Paste" from the Edit menu.
	*/
	public void handlePaste( ActionEvent event ) {
	    try {
	        clipboard.paste();
	    } catch (Exception e) {
	        //  Simply ignore any attempt to paste bad data.
	    }
	}
	
	/**
	*  Handles the user choosing "Save To CSV..." from the File menu.
	*  Allows the user to save the table to disk as a CSV file.
	*/
	public void handleSaveAs( ActionEvent event ) {
	
		// Does this data already have a file name/path associated with it?
        String fileName = "Untitled";
        String dir = null;
        if (lastFile != null) {
            dir = lastFile.getParent();
            fileName = lastFile.getName();
        }
        int idx = fileName.lastIndexOf(".");
        if (idx > 0)
            fileName = fileName.substring(0, idx);
        fileName += "." + EXTENSION;
        
        //	Ask the user to select a file for saving.
        ExtFilenameFilter filter = new ExtFilenameFilter(EXTENSION, "Comma-Separated-Value table file");
        String fileExistsMsgFmt = "<html>A file with the name \"{0}\" already exists.<br>Do you want to replace it?</html>";
        File chosenFile = AppUtilities.selectFile4Save(TableViewWindow.this,
                "Choose a name for this data table file", dir, fileName, filter,
                EXTENSION, fileExistsMsgFmt, "Warning");

        if (chosenFile != null) {
            if (chosenFile.exists() && !chosenFile.canWrite()) {
                String canNotWrite2File = "<html>Can not write to the selected file.<br>Make sure it is not locked or in use by another program.</html>";
                JOptionPane.showMessageDialog(TableViewWindow.this,
                        canNotWrite2File, "I/O Error", JOptionPane.ERROR_MESSAGE);
                return;
            }
            
			//	Get a CSV file reader/writer to write out the file.
			FTableReader writer = CSV_READER;
			
			//	Create a table database to save.
			FTableList db = FTableList.newInstance();
            db.add(table);
						
			//	Add a comment to the table database saying how it was created.
			db.addNote(table.toString());
			StringBuilder buffer = new StringBuilder("Date: ");
			Date theDate = new Date();
			buffer.append(DateFormat.getDateInstance( DateFormat.SHORT ).format( theDate ));
			buffer.append(", ");
			buffer.append(DateFormat.getTimeInstance().format( theDate ));
			db.addNote(buffer.toString());
			
			//	Write out the file.
			try (FileOutputStream os = new FileOutputStream(chosenFile)) {
				writer.write(os, db );
                
			} catch (IOException e) {
				JOptionPane.showMessageDialog( this, e.toString(), "I/O Error",
						JOptionPane.ERROR_MESSAGE );
			}
		}
	}
	
	/**
	*  Handle the user choosing "Close" from the File menu.  This implementation
	*  dispatches a "Window Closing" event to this window.
	*/
	public void handleClose( ActionEvent event ) {
		this.dispatchEvent( new WindowEvent( this, WindowEvent.WINDOW_CLOSING ) );
	}


    /**
    *  A model for our table that figures out what value to put
    *  in what cell.
    */
	private class ViewTableModel extends AbstractTableModel {
	    
        // These methods always need to be implemented.
        @Override
        public int getColumnCount() { return names.length; }
        @Override
        public int getRowCount() { return table.size(); }
        
        @Override
        public Object getValueAt(int row, int col) {
        	String value;

        	int numDims = table.getDimensions();
        	if (col == numDims) {
                //  We are dealing with the dependent value.
        	    
        	    //  Determine position in the table.
        	    pos = tablePosition(row, numDims, pos);

        	    //  Get the value from the table.
        		value = nfd.format(table.get(pos));
        		
        	} else {
        	    //  We are dealing with one of the independents.

                //  Determine position in the table.
        	    int idx = indepPosition(row, col);

        	    //  Get the value from the table.
        		value = nfi.format(table.getBreakpoint(col, idx));
        	}

        	return value;
        }

        // The default implementations of these methods in
        // AbstractTableModel would work, but we can refine them.
        @Override
        public String getColumnName(int column) {return names[column];}
        @Override
        public Class getColumnClass(int col) {return getValueAt(0,col).getClass();}
        @Override
        public boolean isCellEditable(int row, int col) {
            boolean value = true;
            if (col < table.getDimensions())
                value = false;
            return value;
        }
        @Override
        public void setValueAt(Object aValue, int row, int col) {
        	try {
        		
            	int numDims = table.getDimensions();
            	if (col == numDims) {
                    //  We are dealing with the dependent value.
        	    
        		    float value = nfd.parse((String)aValue).floatValue();
        		    
            	    //  Determine position in the table.
            	    pos = tablePosition(row, numDims, pos);

                    //  Set the dependent value.
                    table.set(pos, value);

            	} else {
            	    //  We are dealing with one of the independents.

        		    float value = nfi.parse((String)aValue).floatValue();
        		    
                    //  Determine position in the table.
            	    int idx = indepPosition(row, col);

                    //  Set the independent value.
        			table.setBreakpoint(col, idx, value);
        		}
        		
        	} catch (ParseException e) {
        		JOptionPane.showMessageDialog( TableViewWindow.this, "Illegal input, please try again.",
							"Parsing Error", JOptionPane.ERROR_MESSAGE );
				TableViewWindow.this.repaint();
        	}
        	
        }


        private List<Integer64> tablePosition(int row, int numDims, List<Integer64> pos) {
            
            int numElements = table.size();
    	    for (int i=0; i < numDims; ++i) {
    		    numElements /= table.getNumBreakpoints(i);

    		    int idx = 0;
    		    if (numElements > 1) {
    	            int diff = row - numElements;
    	            while (diff >= 0) {
    	                diff -= numElements;
    	                ++idx;
    	            }

    	        } else
    	            idx = row;
                pos.set(i, Integer64.valueOf(idx));
    		    row -= idx*numElements;
    	    }

    	    return pos;
        }

        private int indepPosition(int row, int dimension) {
            int numDim = table.getDimensions();
            int denom = 1;
            for (int i=dimension + 1; i < numDim; ++i)
                denom *= table.getNumBreakpoints(i);


            int offset = 0;
            for (int i=0; i < numDim; ++i) {
                if (i != dimension) {
                    int prod = 1;
                    for (int k=i+1; k < numDim; ++k)
                        prod *= table.getNumBreakpoints(k);

                    if (prod != 1)
                        offset += (int)(row/prod)*prod;
                }
            }

            int pos = (row - offset)/denom;                
            return pos;
        }
        
     }
     
}
