/*
 *  TestConditionsDialog  -- A dialog for defining test conditions in AeroLS.
 *
 *   Copyright (C) 2002-2024 by Joseph A. Huwaldt
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.js.ParamTools.Param;
import jahuwaldt.js.ParamTools.ParamInput;
import jahuwaldt.swing.DecimalField;
import jahuwaldt.swing.EscapeJDialog;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.*;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Hashtable;
import javax.measure.quantity.Angle;
import javax.measure.unit.NonSI;
import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;

/**
 * A dialog the allows the user to enter/change the values of the test condition
 * break points.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: June 11, 2002
 * @version January 21, 2024
 */
public class TestConditionsDialog extends EscapeJDialog {

    //  Lists of independent break points.
    private ArrayList[] BPlists = {new ArrayList(), new ArrayList()};

    //  The project data.
    private ProjectData pdata;
    private int machColumn;

    //  The table used by this dialog to display the data.
    private final JTable table;
    private BPTableModel tableModel;

    //  Input text field for angle of sideslip.
    ParamInput<Angle> betaInput;

    /**
     * Construct a dialog that allows the user to enter/change the test
     * condition break points.
     *
     * @param parent A reference to the main window that this dialog belongs to.
     * @param title The title for the dialog window.
     * @param pdata The project data for this model.
     */
    @SuppressWarnings("OverridableMethodCallInConstructor")
    public TestConditionsDialog(JFrame parent, String title, ProjectData pdata) {
        super(parent, title);

        this.pdata = pdata;

        //  Create lists of breakpoints from the arrays of breakpoints input.
        Hashtable<String, double[]> tc = pdata.testConditions;
        String[] columnNames = pdata.indepNames;
        int numLists = columnNames.length;
        for (int i = 0; i < numLists; ++i) {
            String key = columnNames[i];
            if (key.equals("Mach"))
                machColumn = i;
            double[] bp = tc.get(key);

            int length = bp.length;
            for (int j = 0; j < length; ++j)
                BPlists[i].add(bp[j]);
        }

        //  Layout the dialog window.
        Container cp = this.getContentPane();
        cp.setLayout(new BorderLayout());

        //  Create an instructions panel.
        JPanel instructions = new JPanel(new BorderLayout());
        instructions.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));
        cp.add(instructions, BorderLayout.NORTH);
        JLabel label = new JLabel("Input breakpoints for Mach number and angle of attack.");
        instructions.add(label, BorderLayout.NORTH);
        label = new JLabel("Each AOA is calculated at all Mach breakpoints.");
        instructions.add(label, BorderLayout.SOUTH);

        //  Create a panel to contain our table and related items.
        JPanel inputs = new JPanel(new BorderLayout());
        cp.add(inputs, BorderLayout.CENTER);

        //  Create a table that displays our breakpoints.
        tableModel = new BPTableModel();
        final DecimalField decField = new DecimalField(7, (DecimalFormat) NumberFormat.getInstance());
        final DefaultCellEditor decEditor = new DefaultCellEditor(decField) {
            //Override DefaultCellEditor's getCellEditorValue method
            //to return a Double, not a String:
            @Override
            public Object getCellEditorValue() {
                return decField.getValue();
            }
        };
        table = new JTable(tableModel) {
            //  Override getCellEditor to return the decimal field editor
            //  if the user is editing a value in one of the tables and
            //  the regular cell editor otherwise.
            @Override
            public TableCellEditor getCellEditor(int row, int column) {
                if (row < BPlists[column].size())
                    return decEditor;
                return super.getCellEditor(row, column);
            }
        };

        table.setPreferredScrollableViewportSize(new Dimension(100, 150));
        table.setCellSelectionEnabled(true);

        // Create the scroll pane and add the table to it. 
        JScrollPane scrollPane = new JScrollPane(table);
        scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        inputs.add(scrollPane, BorderLayout.CENTER);

        //  Create buttons for editing the table.
        Box box = Box.createHorizontalBox();
        inputs.add(box, BorderLayout.NORTH);
        JButton button = new JButton("Insert");
        button.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                tableModel.insertCell();
            }
        });
        box.add(button);
        box.add(Box.createHorizontalStrut(6));

        button = new JButton("Delete");
        button.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                tableModel.deleteCell();
            }
        });
        box.add(button);
        box.add(Box.createGlue());

        //  Define a panel for scalar inputs.
        JPanel scalars = new JPanel();
        scalars.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));
        inputs.add(scalars, BorderLayout.SOUTH);
        box = Box.createHorizontalBox();
        scalars.add(box);
        box.add(Box.createGlue());

        //  Create sideslip inputs.
        Param<Angle> aParam = new Param("Sideslip", pdata.beta, NonSI.DEGREE_ANGLE);
        aParam.setAvailUnits(ProjectData.ANGULAR_UNITS);
        betaInput = new ParamInput(aParam);
        box.add(betaInput);
        box.add(Box.createGlue());

        //  Define OK and Cancel buttons.
        box = Box.createHorizontalBox();
        box.add(Box.createGlue());

        JButton cancelBtn = new JButton("Cancel");
        cancelBtn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                TestConditionsDialog.this.performEscapeAction(null);
            }
        });
        box.add(cancelBtn);

        box.add(Box.createHorizontalStrut(40));

        JButton okayBtn = new JButton("Done");
        okayBtn.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                handleDone();
            }
        });
        box.add(okayBtn);

        box.add(Box.createHorizontalStrut(20));
        cp.add(box, BorderLayout.SOUTH);

        // Define a window listener that handles closing the window.
        this.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                TestConditionsDialog.this.handleWindowClosing(e);
            }
        });

    }

    /**
     * Handles the user clicking on the "Done" button. Updates the project data
     * structure's test conditions to match those in the table.
     */
    private void handleDone() {

        //  Create new arrays of breakpoints.
        int numLists = BPlists.length;
        for (int j = 0; j < numLists; ++j) {

            int length = BPlists[j].size();
            double[] bp = new double[length];
            for (int i = 0; i < length; ++i)
                bp[i] = ((Double) BPlists[j].get(i));

            //  Update the project data test conditions.
            pdata.testConditions.put(pdata.indepNames[j], bp);
        }

        //  Extract the angle of sideslip.
        pdata.beta = betaInput.get().doubleValue(NonSI.DEGREE_ANGLE);

        //  Close the window and dispose of it.
        setVisible(false);
        dispose();

    }

    /**
     * Handles the user clicking on the "close" box.
     */
    public synchronized void handleWindowClosing(WindowEvent e) {
        performEscapeAction(null);
    }

    /**
     * Response to ESCAPE key pressed goes here. Close the window without
     * changing anything.
     */
    @Override
    protected void performEscapeAction(KeyEvent e) {

        //  Close the window and dispose of it.
        setVisible(false);
        dispose();
    }

    /**
     * This is the data model for our table. It supplies the table with the
     * data, properly formatted, and puts any data that the user enters into the
     * table into our internal data structures.
     */
    private class BPTableModel extends AbstractTableModel {

        @Override
        public String getColumnName(int col) {
            return pdata.indepNames[col];
        }

        @Override
        public int getRowCount() {
            int max = 0;

            int numLists = BPlists.length;
            for (int i = 0; i < numLists; ++i)
                max = Math.max(max, BPlists[i].size());

            return max + 1;
        }

        @Override
        public int getColumnCount() {
            return pdata.indepNames.length;
        }

        @Override
        public Object getValueAt(int row, int col) {
            Object value = "";

            if (row < BPlists[col].size())
                value = BPlists[col].get(row);

            return value;
        }

        @Override
        public boolean isCellEditable(int row, int col) {
            return true;
        }

        @Override
        public void setValueAt(Object value, int row, int col) {
            try {
                Double data;
                if (value instanceof Double)
                    data = (Double) value;
                else
                    data = Double.valueOf((String) value);

                if (col == machColumn) {
                    if (data >= 1) {
                        JOptionPane.showMessageDialog(TestConditionsDialog.this,
                                "Mach number must be subsonic!", "Input Error",
                                JOptionPane.ERROR_MESSAGE);
                        return;
                    }
                }

                if (row < BPlists[col].size()) {
                    BPlists[col].remove(row);
                    BPlists[col].add(row, data);

                } else {
                    BPlists[col].add(data);
                    row = BPlists[col].size() - 1;
                }

                fireTableCellUpdated(row, col);

            } catch (Exception e) {
                JOptionPane.showMessageDialog(TestConditionsDialog.this,
                        "Invalid input, please try again.", "Input Error",
                        JOptionPane.ERROR_MESSAGE);
                TestConditionsDialog.this.table.repaint();
            }
        }

        /**
         * Method that deletes the currently selected cell.
         */
        public void deleteCell() {
            int row = table.getSelectedRow();
            int col = table.getSelectedColumn();

            if (row > -1 && col > -1) {
                BPlists[col].remove(row);

                fireTableDataChanged();
            }

        }

        /**
         * Method that inserts a new cell.
         */
        public void insertCell() {
            int row = table.getSelectedRow();
            int col = table.getSelectedColumn();

            if (row > -1 && col > -1) {
                BPlists[col].add(row, 0);

                fireTableDataChanged();

                ListSelectionModel sel = table.getSelectionModel();
                sel.setLeadSelectionIndex(row);
                table.setEditingRow(row);
                table.setEditingColumn(col);
            }
        }

    }

}
