/*
 *  ViewPanel  -- A panel that draws a 2D view of the geometry in AeroLS.
 *
 *  Copyright (C) 2002-2024, Joseph A. Huwaldt
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/gpl.html
 */

import jahuwaldt.aero.panelgeom.GeomComponent;
import jahuwaldt.aero.panelgeom.GeomNetwork;
import jahuwaldt.aero.panelgeom.GeomVehicle;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import javax.swing.JPanel;

/**
 * A panel component that draws a view of the specified geometry.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: May 29, 2002
 * @version January 21, 2024
 */
public abstract class ViewPanel extends JPanel {

    //  The number of pixels or dots per meter (if drawn full scale) (dots/in*in/ft*ft/m).
    protected static final float DPM = 72F * 12F / 0.3048F;

    //  The current drawing scale.
    private int modelScaleDenom = 100;
    protected float scale = DPM / modelScaleDenom;

    //  The pens we are using to draw with.
    protected Stroke thinLine = new BasicStroke(0.00035F * modelScaleDenom);
    protected Stroke thickLine = new BasicStroke(0.0007F * modelScaleDenom);
    private final float[] clDash = {0.038F * modelScaleDenom, 0.0032F * modelScaleDenom,
                                    0.0068F * modelScaleDenom, 0.0032F * modelScaleDenom};
    protected Stroke centerLine = new BasicStroke(0.00035F * modelScaleDenom, BasicStroke.CAP_ROUND,
                                                    BasicStroke.JOIN_BEVEL, 1, clDash, 0);

    //  The geometry we are drawing.
    protected GeomVehicle vehicle;

    /**
     * The constructor for this class.
     */
    @SuppressWarnings("OverridableMethodCallInConstructor")
    protected ViewPanel(GeomVehicle geom) {
        vehicle = geom;
        this.setBackground(Color.white);
    }

    /**
     * This method sets the model drawing scale.
     *
     * @param denom The denominator of the scale factor (e.g.: 100 for 1/100
     * scale).
     */
    public void setScale(int denom) {

        modelScaleDenom = denom;
        scale = DPM / modelScaleDenom;
        thinLine = new BasicStroke(0.00035F * denom);
        thickLine = new BasicStroke(0.0007F * denom);
        clDash[0] = 0.0038F * denom;
        clDash[1] = 0.0032F * denom;
        clDash[2] = 0.0068F * denom;
        clDash[3] = 0.0032F * denom;
        centerLine = new BasicStroke(0.00035F * denom, BasicStroke.CAP_ROUND,
                BasicStroke.JOIN_BEVEL, 1, clDash, 0);
    }

    /**
     * Method that returns the denominator of the current model drawing scale.
     * For example, if the current scale is 1/200, then 200 is returned.
     */
    public int getScale() {
        return modelScaleDenom;
    }

    /**
     * Method used to change the geometry viewed by this panel.
     */
    public void setGeometry(GeomVehicle geom) {
        vehicle = geom;
        this.repaint();
    }

    /**
     * Called automatically by Swing system to paint this component.
     */
    @Override
    public void paintComponent(Graphics gc) {

        // Paint the background
        super.paintComponent(gc);

        Graphics2D g2 = (Graphics2D) gc;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        //  Draw a string describing the view.
        Dimension size = this.getSize();
        drawViewName(g2, size);

        //  Draw the scale factor.
        drawScaleFactor(g2, size);

        //  Save states that will need to be restored later.
        Stroke saveStroke = g2.getStroke();
        AffineTransform saveXform = g2.getTransform();

        //  Set up drawing transformations.
        AffineTransform transform = getTransform(size);
        g2.transform(transform);

        //  Render the geometry.
        renderWireFrameVehicle(g2, vehicle);

        //  Draw plane of symmetry line.
        g2.setStroke(centerLine);
        g2.draw(createSymmetryLine());

        //  Be kind, rewind.
        g2.setTransform(saveXform);
        g2.setStroke(saveStroke);
    }

    /**
     * Method that returns an appropriate transformation for this view panel.
     * Typically, this involves a translation to center the model in the panel.
     */
    protected abstract AffineTransform getTransform(Dimension panelSize);

    /**
     * Method that renders a wire frame model of a single network with an
     * appropriate view orientation.
     */
    protected abstract void renderWFNetwork(Graphics2D gc, GeomNetwork network);

    /**
     * Method that returns a line representing the plane of symmetry for this
     * view.
     */
    protected abstract Line2D createSymmetryLine();

    /**
     * Method that returns the name of this view. It should return, "Top View",
     * "Side View", or "Bottom View".
     */
    protected abstract String getViewName();

    /**
     * Method that renders the model geometry as a wire-frame.
     */
    private void renderWireFrameVehicle(Graphics2D gc, GeomVehicle vehicle) {

        int size = vehicle.size();
        if (size > 0) {

            // Loop over all the components in each vehicle.
            for (int i = 0; i < size; ++i) {
                GeomComponent comp = (GeomComponent) vehicle.get(i);

                // Loop over all the networks in each component.
                int numNets = comp.size();
                for (int j = 0; j < numNets; ++j) {
                    GeomNetwork net = (GeomNetwork) comp.get(j);
                    renderWFNetwork(gc, net);
                }
            }
        }
    }

    /**
     * Method that draws the view name to the bottom of the view.
     */
    private void drawViewName(Graphics2D g2, Dimension size) {

        int w = size.width;
        int h = size.height;
        String name = getViewName();

        FontMetrics metrics = g2.getFontMetrics();
        int width = metrics.stringWidth(name);

        g2.drawString(name, w / 2 - width / 2, h - 5);

    }

    /**
     * Method that draws the scale factor (e.g.: 1/200) to the upper right hand
     * corner of the view panel.
     */
    private void drawScaleFactor(Graphics2D g2, Dimension size) {

        int w = size.width;
        //int h = size.height;
        String scl = "1/" + getScale();

        FontMetrics metrics = g2.getFontMetrics();
        int width = metrics.stringWidth(scl);
        int height = metrics.getHeight();

        g2.drawString(scl, w - 20 - width / 2, 20 + height / 2);

    }

}
