/*
 *  dihedfn_b_zrle_ztle - Method for calculating dihedral as a function of span, zrle, & ztle.
 *
 *  Copyright (C) 2002 - 2015 by Joseph A. Huwaldt
 *  All rights reserved.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.js.ParamTools.AbstractParamMethod;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Angle;
import javax.measure.unit.SI;

/**
 * Calculates dihedral as a function of wing span and the Z locations of the LE
 * of the root and tip chords.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt Date: May 23, 2002
 * @version June 19, 2015
 */
public class dihedfn_b_zrle_ztle extends AbstractParamMethod<Angle> {

    /**
     * Constructor for this method.
     *
     * @param b Label for the "wing span" parameter.
     * @param xrle Label for the "X coord of root chord leading edge" parameter.
     * @param xtle Label for the "X coord of tip chord leading edge" parameter.
     */
    public dihedfn_b_zrle_ztle(String b, String xrle, String xtle) {
        _inputLabels = new String[3];
        _inputLabels[0] = b;
        _inputLabels[1] = xrle;
        _inputLabels[2] = xtle;
    }

    /**
     * Calculates dihedral as a function of wing span and the Z locations of the
     * LE of the root and tip chords. Method:
     *      dihedral = atan( (xtle-xrle)/b )
     *
     * @param inputs Array of input parameters. [0=wing span (b), 1=X location
     *  of root chord LE (xrle), 2=X location of tip chord LE (xtle),].
     * @return Wing dihedral angle in angular units.
     */
    @Override
    public Parameter<Angle> calc(Parameter[] inputs) {
        double b = inputs[0].doubleValue(SI.METER);
        double xrle = inputs[1].doubleValue(SI.METER);
        double xtle = inputs[2].doubleValue(SI.METER);
        double dihed = Math.atan((xtle - xrle) / b);
        return Parameter.valueOf(dihed, SI.RADIAN);
    }

}
