/*
 *   ARfn_sweepLE_sweepTE_taper - Method that calculates AR using sweepLE, sweepTE, and taper.
 *
 *   Copyright (C) 2011-2016, by Joseph A. Huwaldt,
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.pmethods.winggeom;

import jahuwaldt.js.ParamTools.*;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Dimensionless;
import javax.measure.unit.SI;

/**
 * Calculates planform wing aspect ratio based on leading edge sweep angle, trailing edge
 * sweep angle and wing taper ratio.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: September 10, 2011
 * @version March 26, 2016
 */
@SuppressWarnings("serial")
public class ARfn_sweepLE_sweepTE_taper extends AbstractParamMethod<Dimensionless> {

    /**
     * Constructor for this method.
     *
     * @param sweepLE Label for the "leading edge sweep" parameter.
     * @param sweepTE Label for the "trailing edge sweep" parameter.
     * @param taper   Label for the "taper ratio" parameter.
     */
    public ARfn_sweepLE_sweepTE_taper(String sweepLE, String sweepTE, String taper) {
        _inputLabels = new String[3];
        _inputLabels[0] = sweepLE;
        _inputLabels[1] = sweepTE;
        _inputLabels[2] = taper;
    }

    /**
     * Calculates planform wing aspect ratio based on leading edge sweep angle, quarter
     * chord sweep angle and wing taper ratio.
     * <p>
     * Method: <code>AR = 4*(1 - taper)/(1 + taper)/(tan(sweepLE) - tan(sweepTE))</code>
     * </p>
     *
     * @param inputs Array of input parameters. [0=leading edge sweep angle in rad
     *               (sweepLE), 1=trailing edge sweep angle in rad (sweepTE), 2=wing taper
     *               ratio (taper)].
     * @return Planform aspect ratio (AR).
     */
    @Override
    public Parameter<Dimensionless> calc(Parameter[] inputs) {
        double sweepLE = inputs[0].doubleValue(SI.RADIAN);
        double sweepTE = inputs[1].doubleValue(SI.RADIAN);
        double taper = inputs[2].getValue();
        double AR = 4 * (1 - taper) / (1 + taper) / (Math.tan(sweepLE) - Math.tan(sweepTE));
        return Parameter.valueOf(AR, Dimensionless.UNIT);
    }

}
