/*
 *   bfn_S_Cr_Ct - Method for calculating b as a function of S, Cr and Ct.
 *
 *   Copyright (C) 1998-2016, by Joseph A. Huwaldt.
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.pmethods.winggeom;

import jahuwaldt.js.ParamTools.*;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Length;

/**
 * Calculates planform wing span based on planform area, wing root chord length, and wing
 * tip chord length.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: February 6, 1999
 * @version March 26, 2016
 */
@SuppressWarnings("serial")
public class bfn_S_Cr_Ct extends AbstractParamMethod<Length> {

    /**
     * Constructor for this method.
     *
     * @param S  Label for the "planform area" parameter.
     * @param Cr Label for the "root chord" parameter.
     * @param Ct Label for the "tip chord" parameter.
     */
    public bfn_S_Cr_Ct(String S, String Cr, String Ct) {
        _inputLabels = new String[3];
        _inputLabels[0] = S;
        _inputLabels[1] = Cr;
        _inputLabels[2] = Ct;
    }

    /**
     * Calculates planform wing span based on planform wing area, wing root chord length,
     * and wing tip chord length.
     * <p>
     * Method: <code>b = 2*S/(Cr + Ct)</code>
     * </p>
     *
     * @param inputs Array of input parameters. [0=planform area (S), 1=wing root chord
     *               length (Cr), 2=wing tip chord length (Ct)].
     * @return Planform wing span (b) in base length units.
     */
    @Override
    public Parameter<Length> calc(Parameter[] inputs) {
        Parameter S = inputs[0];
        Parameter Cr = inputs[1];
        Parameter Ct = inputs[2];
        Parameter b = S.times(2).divide(Cr.plus(Ct));   //  b = 2*S/(Cr + Ct)
        return b.asType(Length.class);
    }

}
