/*
*   etaMACfn_yMAC_b - Method for calculating etaMAC as a function of yMAC and b.
*
*   Copyright (C) 1998-2009 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.winggeom;


import javax.measure.quantity.Dimensionless;
import javax.measure.unit.SI;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  <p>  Calculates fractional spanwise location of MAC (Cbar) based on
*       spanwise location of MAC and wing span.
*  </p>
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  February 6, 1999
*  @version  April 26, 2009
**/
public class etaMACfn_yMAC_b extends AbstractParamMethod<Dimensionless> {

	/**
	*  Constructor for this method.
	*
	*  @param  yMAC  Label for the "spanwise location of MAC" parameter.
	*  @param  b     Label for the "span" parameter.
	**/
	public etaMACfn_yMAC_b( String yMAC, String b ) {
		_inputLabels = new String [ 2 ];
		_inputLabels[0] = yMAC;
		_inputLabels[1] = b;
	}

	/**
	*  Calculates fractional spanwise location of MAC (Cbar) based on
	*  spanwise location of MAC and wing span.
	*  Method:  etaMAC = 2*yMAC/b
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=spanwise location of MAC (yMAC), 1=wing span (b)].
	*  @return  Spanwise location of the MAC (Cbar) as a fraction of span (b).
	**/
	public Parameter<Dimensionless> calc( Parameter[] inputs ) {
		Parameter yMAC = inputs[0];
		Parameter b = inputs[1];
		Parameter etaMAC = yMAC.times(2).divide(b);		//	etaMAC = 2 * yMAC / b
		return etaMAC.asType(Dimensionless.class);
	}


}


