/*
 *  sweepLEfn_xMAC_yMAC - Method for calculating LE sweep as a function of xMAC and yMAC.
 *
 *   Copyright (C) 1998-2016, by Joseph A. Huwaldt.
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.pmethods.winggeom;

import jahuwaldt.js.ParamTools.*;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Angle;
import javax.measure.unit.SI;

/**
 * Calculates leading edge wing sweep angle based on the spanwise and longitudinal
 * location of the leading edge of the mean aerodynamic chord (Cbar).
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: February 7, 1999
 * @version March 26, 2016
 */
@SuppressWarnings("serial")
public class sweepLEfn_xMAC_yMAC extends AbstractParamMethod<Angle> {

    /**
     * Constructor for this method.
     *
     * @param xMAC Label for the "longitudinal position of MAC" parameter.
     * @param yMAC Label for the "spanwise position of MAC" parameter.
     */
    public sweepLEfn_xMAC_yMAC(String xMAC, String yMAC) {
        _inputLabels = new String[2];
        _inputLabels[0] = xMAC;
        _inputLabels[1] = yMAC;
    }

    /**
     * Calculates leading edge wing sweep angle based on the spanwise and longitudinal
     * location of the leading edge of the mean aerodynamic chord (Cbar).
     * <p>
     * Method: <code>sweepLE = atan(xMAC/yMAC)</code>
     * </p>
     *
     * @param inputs Array of input parameters. [0=longitudinal position of leading edge
     *               of MAC (xMAC), 1=spanwise location of MAC (yMAC)].
     * @return Wing sweep angle at the leading edge in base angular units (radians).
     */
    @Override
    public Parameter<Angle> calc(Parameter[] inputs) {
        double xMAC = inputs[0].getValue();
        double yMAC = inputs[1].getValue();
        double sweepLE = Math.atan(xMAC / yMAC);
        return Parameter.valueOf(sweepLE, SI.RADIAN);
    }

}
