/*
 *   taperfn_Cr_Ct - Method for calculating taper as a function of Cr and Ct.
 *
 *   Copyright (C) 1998-2016, by Joseph A. Huwaldt.
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.pmethods.winggeom;

import jahuwaldt.js.ParamTools.*;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Dimensionless;

/**
 * Calculates wing taper ratio based on wing root and tip chord lengths.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: February 6, 1999
 * @version March 26, 2016
 */
@SuppressWarnings("serial")
public class taperfn_Cr_Ct extends AbstractParamMethod<Dimensionless> {

    /**
     * Constructor for this method.
     *
     * @param Cr Label for the "root chord" parameter.
     * @param Ct Label for the "tip chord" parameter.
     */
    public taperfn_Cr_Ct(String Cr, String Ct) {
        _inputLabels = new String[2];
        _inputLabels[0] = Cr;
        _inputLabels[1] = Ct;
    }

    /**
     * Calculates wing taper ratio based on wing root and tip chord lengths.
     * <p>
     * Method: <code>taper = Ct/Cr</code>
     * </p>
     *
     * @param inputs Array of input parameters. [0=root chord length (Cr), 1=tip chord
     *               length (Ct)].
     * @return Wing taper ratio (taper).
     */
    @Override
    public Parameter<Dimensionless> calc(Parameter[] inputs) {
        Parameter Cr = inputs[0];
        Parameter Ct = inputs[1];
        Parameter taper = Ct.divide(Cr);		//	taper = Ct / Cr;
        return taper.asType(Dimensionless.class);
    }

}
