/*
 *   taperfn_etaMAC - Method for calculating taper as a function of etaMAC.
 *
 *   Copyright (C) 1998-2016, by Joseph A. Huwaldt.
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.pmethods.winggeom;

import jahuwaldt.js.ParamTools.*;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Dimensionless;

/**
 * Calculates wing taper ratio based on fractional spanwise location of the MAC (Cbar).
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: February 6, 1999
 * @version March 26, 2016
 */
@SuppressWarnings("serial")
public class taperfn_etaMAC extends AbstractParamMethod<Dimensionless> {

    /**
     * Constructor for this method.
     *
     * @param etaMAC Label for the "fraction spanwise location of MAC" parameter.
     */
    public taperfn_etaMAC(String etaMAC) {
        _inputLabels = new String[1];
        _inputLabels[0] = etaMAC;
    }

    /**
     * Calculates wing taper ratio based on wing root chord and mean aerodynamic chord
     * length.
     * <p>
     * Method: <code>taper = (3*etaMAC - 1)/(2 - 3*etaMAC)</code>
     * </p>
     *
     * @param inputs Array of input parameters. [0=fractional spanwise location of the MAC
     *               (etaMAC)].
     * @return Wing taper ratio (taper).
     */
    @Override
    public Parameter<Dimensionless> calc(Parameter[] inputs) {
        double etaMAC = inputs[0].getValue();
        double taper = (3 * etaMAC - 1) / (2 - 3 * etaMAC);
        return Parameter.valueOf(taper, Dimensionless.UNIT);
    }

}
