/*
 *   xMACfn_yMAC_sweepLE - Method for calculating xMAC as a function of yMAC and sweepLE.
 *
 *   Copyright (C) 1998-2016, by Joseph A. Huwaldt.
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.pmethods.winggeom;

import jahuwaldt.js.ParamTools.*;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Length;
import javax.measure.unit.SI;

/**
 * Calculates longitudinal location of MAC (Cbar) from the wing apex based on spanwise
 * location of MAC and wing leading edge sweep angle.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: February 7, 1999
 * @version March 26, 2016
 */
@SuppressWarnings("serial")
public class xMACfn_yMAC_sweepLE extends AbstractParamMethod<Length> {

    /**
     * Constructor for this method.
     *
     * @param yMAC    Label for the "spanwise location of MAC" parameter.
     * @param sweepLE Label for the "sweep of the leading edge" parameter.
     */
    public xMACfn_yMAC_sweepLE(String yMAC, String sweepLE) {
        _inputLabels = new String[2];
        _inputLabels[0] = yMAC;
        _inputLabels[1] = sweepLE;
    }

    /**
     * Calculates longitudinal location of MAC (Cbar) from the wing apex based on spanwise
     * location of MAC and wing leading edge sweep angle.
     * <p>
     * Method: <code>xMAC = yMAC*tan(sweepLE)</code>
     * </p>
     *
     * @param inputs Array of input parameters. [0=spanwise locatin of MAC (yMAC), 1=wing
     *               leading edge sweep (sweepLE)].
     * @return Longitudinal location of the MAC (Cbar) from wing apex in base units of
     *         length.
     */
    @Override
    public Parameter<Length> calc(Parameter[] inputs) {
        double yMAC = inputs[0].doubleValue(SI.METER);
        double sweepLE = inputs[1].doubleValue(SI.RADIAN);
        double xMAC = yMAC * Math.tan(sweepLE);
        return Parameter.valueOf(xMAC, SI.METER).to(inputs[0].getUnit());
    }

}
