/*
 *   yMACfn_xMAC_sweepLE - Method for calculating yMAC as a function of xMAC and sweepLE.
 *
 *   Copyright (C) 1998-2016, by Joseph A. Huwaldt.
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.pmethods.winggeom;

import jahuwaldt.js.ParamTools.*;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Length;

/**
 * Calculates spanwise location of MAC (Cbar) based on the longitudinal location of MAC
 * from wing apex and wing leading edge sweep angle.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: February 7, 1999
 * @version March 26, 2016
 */
@SuppressWarnings("serial")
public class yMACfn_xMAC_sweepLE extends AbstractParamMethod<Length> {

    /**
     * Constructor for this method.
     *
     * @param xMAC    Label for the "longitudinal location of MAC" parameter.
     * @param sweepLE Label for the "sweep of the leading edge" parameter.
     */
    public yMACfn_xMAC_sweepLE(String xMAC, String sweepLE) {
        _inputLabels = new String[2];
        _inputLabels[0] = xMAC;
        _inputLabels[1] = sweepLE;
    }

    /**
     * Calculates spanwise location of MAC (Cbar) based on the longitudinal location of
     * MAC from wing apex and wing leading edge sweep angle.
     * <p>
     * Method: <code>yMAC = xMAC/tan(sweepLE)</code>
     * </p>
     *
     * @param inputs Array of input parameters. [0=longitudinal locatin of MAC (xMAC),
     *               1=wing leading edge sweep angle (sweepLE)].
     * @return Spanwise location of the MAC (Cbar) in base units of length.
     */
    @Override
    public Parameter<Length> calc(Parameter[] inputs) {
        Parameter xMAC = inputs[0];
        Parameter sweepLE = inputs[1];
        Parameter yMAC = xMAC.divide(Parameter.tan(sweepLE));	//	yMAC = xMAC / Math.tan( sweepLE );
        return yMAC.asType(Length.class);
    }

}
