/*
 *   FDAReader   -- A class that can read and write FDA formatted multi-table files.
 *
 *   Copyright (C) 2000-2024, by Joseph A. Huwaldt.
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.tables;

import jahuwaldt.io.FileUtils;
import static jahuwaldt.js.tables.FTableReader.RESOURCES;
import jahuwaldt.js.util.TextTokenizer;
import java.io.*;
import java.text.DateFormat;
import java.text.MessageFormat;
import static java.text.MessageFormat.format;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.NoSuchElementException;
import static java.util.Objects.requireNonNull;
import javolution.text.Text;
import javolution.text.TextBuilder;
import javolution.text.TypeFormat;
import javolution.util.FastTable;
import org.jscience.mathematics.number.Float64;
import org.jscience.mathematics.number.Integer64;
import org.jscience.mathematics.vector.Float64Vector;

/**
 * A class that provides a method for reading or writing an FDA (Functional Data Handling
 * system ASCII) multi-table file. FDA is one of the most comprehensive table file formats
 * that can store any number of tables, with each table independently having any number of
 * dimensions and with file level and table level notes.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: April 3, 2000
 * @version January 21, 2024
 */
public class FDAReader implements FTableReader {

    //  The preferred file extension for files of this reader's type.
    private static final String EXTENSION = "fda";

    //  The comment characters used in FDA files.
    private static final String COMMENT1 = "*";
    private static final String COMMENT2 = ";";

    private static final Text LOGICAL = Text.intern("logical");
    private static final Text SPACE = Text.intern(" ");

    /**
     * The number of digits used when formatting a number for writing out to the FDA
     * format file.
     */
    private static final int NUM_DIGITS = 14;

    /**
     * Returns a string representation of the object. This will return a brief description
     * of the format read by this reader.
     */
    @Override
    public String toString() {
        return RESOURCES.getString("fdaDescription");
    }

    /**
     * Returns the preferred file extension (not including the ".") for files of this
     * reader's type.
     *
     * @return The file extension "fda".
     */
    @Override
    public String getExtension() {
        return EXTENSION;
    }

    /**
     * Method that determines if this reader can read data from the specified input
     * stream. Will return FTableReader.MAYBE if the file name has the extension "fda".
     * Will return FTableReader.YES if, and only if, the stream contains "*FDHS" on the
     * 1st line.
     *
     * @param name  The name of the file.
     * @param input An input stream containing the data to be read. Any methods that read
     *              from this stream must first set a mark and then reset back to that
     *              mark before the method returns (even if it returns with an exception).
     * @return FTableReader.NO if the file is not recognized at all or FTableReader.MAYBE
     *         if the filename extension is "*.fda". Will return FTableReader.YES if the
     *         1st line of the file is "*FDHS".
     */
    @Override
    public int canReadData(String name, BufferedInputStream input) throws IOException {
        requireNonNull(name, MessageFormat.format(RESOURCES.getString("paramNullErr"), "name"));
        
        int response = NO;
        String ext = FileUtils.getExtension(name);
        if (EXTENSION.equals(ext) || "fdhs".equals(ext))
            response = MAYBE;

        //  Mark the current position in the input stream so that we can return to it.
        input.mark(10240);  //  10KB buffer

        try {
            //  Create a buffer to hold the data read in from the file.
            byte[] buffer = new byte[10238];

            //  Read in up to 10k worth of data.
            int byteCount = input.read(buffer, 0, 10238);
            if (byteCount < 0)
                return NO;

            //  Turn the buffer into a (potentially 10k long) string.
            String str = new String(buffer, 0, byteCount);

            //  Convert the string into a line number reader.
            LineNumberReader lnr = new LineNumberReader(new StringReader(str));

            //  Skip any blank lines
            String line = lnr.readLine();
            while (line != null && !line.trim().equals(""))
                line = lnr.readLine();

            //  If the file starts with *FDHS, then we've got the right type.
            if (line != null && line.trim().equals("*FDHS"))
                return YES;

        } finally {
            input.reset();
        }

        return response;
    }

    /**
     * Returns true. This class can write data to a CSV formatted file.
     */
    @Override
    public boolean canWriteData() {
        return true;
    }

    /**
     * Compares this object with the specified object for order based on the
     * <code>toString().compareTo(o.toString())</code> method. Returns a negative integer,
     * zero, or a positive integer as this object is less than, equal to, or greater than
     * the specified object.
     */
    @Override
    public int compareTo(FTableReader o) {
        return this.toString().compareTo(o.toString());
    }

    /**
     * Method that reads in Functional Data Handling ASCII system (FDA) formatted data
     * from the specified input stream and returns that data as an {@link FTableList}
     * object.
     *
     * @param input An input stream containing the FDA formatted data.
     * @return An {@link FTableList} object that contains the data read in from the
     *         specified stream.
     * @throws IOException If there is a problem reading the specified stream.
     */
    @Override
    public FTableList read(InputStream input) throws IOException {
        requireNonNull(input, MessageFormat.format(RESOURCES.getString("paramNullErr"), "input"));
        
        //  FDHS ASCII files are required to be in ASCII with U.S. style number formatting.
        //  Get the default locale and save it.  Then change the default locale to US.
        Locale defLocale = Locale.getDefault();
        Locale.setDefault(Locale.US);

        try {
            // Create a reader that can parse the input stream.
            LineNumberReader reader = new LineNumberReader(new InputStreamReader(input));

            // Create a new (empty) table list.
            FTableList db = FTableList.newInstance();

            // Loop over all the tables in the file, reading them in one after the other.
            reader.mark(10240);
            Text aLine = readLineNoComments(reader);
            while (aLine != null) {
                reader.reset();
                Float64Table table = readFDATable(reader);
                db.add(table);

                //  Is there another table?
                reader.mark(10240);
                aLine = readLineNoComments(reader);
            }

            return db;
            
        } finally {
            //  Restore the default locale.
            Locale.setDefault(defLocale);
        }
    }

    /**
     * Method for writing out all the data stored in the specified list of
     * {@link Float64Table} objects to the specified output stream in FDA format.
     *
     * @param output The output stream to which the data is to be written.
     * @param tables The list of {@link Float64Table} objects to be written out.
     * @throws IOException If there is a problem writing to the specified stream.
     */
    @Override
    public void write(OutputStream output, FTableList tables) throws IOException {
        requireNonNull(output, MessageFormat.format(RESOURCES.getString("paramNullErr"), "output"));
        requireNonNull(tables, MessageFormat.format(RESOURCES.getString("paramNullErr"), "tables"));
        
        if (tables.isEmpty())
            return;

        //  FDHS ASCII files are required to be in ASCII with U.S. style number formatting.
        //  Get the default locale and save it.  Then change the default locale to US.
        Locale defLocale = Locale.getDefault();
        Locale.setDefault(Locale.US);

        try {
            // Get a reference to the output stream writer.
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(output));

            // Identify this as an FDHS formatted file.
            writer.write("*FDHS");
            writer.newLine();

            // Write out the file level notes if there are any.
            writeComments(writer, tables.getAllNotes());

            // Loop over all the tables in the collection and write them out to the file.
            for (Float64Table table : tables)
                writeFDATable(writer, table);

            //  Flush the output.
            writer.flush();
            
        } finally {
            //  Restore the default locale.
            Locale.setDefault(defLocale);
        }
    }

    /**
     * Read a single FDHS table from an FDA formatted file.
     *
     * @param reader A reader that reads in the characters from the input stream.
     * @return A table containing the data read in from the stream.
     */
    private static Float64Table readFDATable(LineNumberReader reader) throws IOException {

        // *** Read in the FDA file header.
        //  Create a list to hold any comments.
        FastTable<Text> comments = FastTable.newInstance();

        // Read in the table name, independent names and 1st number of breakpoints.
        Text aLine = readLineKeepingComments(reader, comments);
        if (aLine == null)
            throw new IOException(format(RESOURCES.getString("EOFErrMsg"), reader.getLineNumber()));
        TextTokenizer tokenizer = TextTokenizer.newInstance();
        tokenizer.setText(aLine);
        FastTable<Text> names = FastTable.newInstance();
        int value = 0;
        int numTokens = tokenizer.countTokens();
        for (int i = 0; i < numTokens; ++i) {
            Text token = tokenizer.nextToken().trim();
            try {
                //  Try formatting the token as a number, that is how we will know when we have
                //  reached the end of the labels.
                value = TypeFormat.parseInt(token);
                break;

            } catch (NumberFormatException e) {
                //  Wasn't a number, so we must have a name.
                names.add(token);
            }
        }

        // Start filling in some variables.
        int numDims = names.size() - 1;
        Text tblName = names.get(0);
        FastTable<Text> indepNames = FastTable.newInstance();
        for (int j = numDims - 1; j >= 0; --j)
            indepNames.add(names.get(j + 1));

        FastTable.recycle(names);

        // Read in the number of breakpoints per dimension
        // (first one has already been read in).
        FastTable<Integer64> elemPerDim = readNumBreakpointsPerDim(reader, tokenizer, numDims, indepNames, value);

        // Skip optional "logical" keyword and go to next line.
        aLine = readLineKeepingComments(reader, comments);
        if (aLine == null)
            throw new IOException(format(RESOURCES.getString("EOFErrMsg"), reader.getLineNumber()));
        if (aLine.toLowerCase().startsWith(LOGICAL)) {
            aLine = readLineKeepingComments(reader, comments);
            if (aLine == null)
                throw new IOException(format(RESOURCES.getString("EOFErrMsg"), reader.getLineNumber()));
        }

        // Skip the frequency card (read to start of next line).
        aLine = readLineKeepingComments(reader, comments);
        if (aLine == null)
            throw new IOException(format(RESOURCES.getString("EOFErrMsg"), reader.getLineNumber()));
        tokenizer.setText(aLine);

        // *** Read in the breakpoints.
        // Read in breakpoints for each independent variable.
        FastTable<Float64Vector> breakpoints = readBreakpoints(reader, tokenizer, elemPerDim, numDims);

        // *** Read in the dependent values.
        // Recursively read in the dependent data arrays.
        Float64NDimArray depData = Float64NDimArray.newInstance(elemPerDim);
        FastTable<Integer64> indep = FastTable.newInstance();
        indep.setSize(numDims);

        rReadFDA(0, reader, indep, depData, elemPerDim);

        // Create a new doubleTable object using this information.
        Float64Table theTable = Float64Table.valueOf(tblName, indepNames, breakpoints, depData);
        theTable.addAllNotes(comments);

        //  Clean up before leaving.
        FastTable.recycle(indepNames);
        FastTable.recycle(elemPerDim);
        FastTable.recycle(breakpoints);
        FastTable.recycle(indep);
        FastTable.recycle(comments);
        Float64NDimArray.recycle(depData);

        return theTable;
    }

    /**
     * Read in the number of breakpoints in each dimension of the data.
     *
     * @param reader     A reader that reads in the characters from the input stream.
     * @param tokenizer  The tokenizer to use with the text set to the 1st line of
     *                   breakpoint data.
     * @param numDims    The number of dimensions to read in breakpoints for.
     * @param indepNames The name of each independent variable (for each dimension).
     * @param firstValue The number of breakpoints in the 1st dimension of the table.
     * @return A list containing the number of breakpoints in each dimension fo the table.
     */
    private static FastTable<Integer64> readNumBreakpointsPerDim(LineNumberReader reader, TextTokenizer tokenizer,
            int numDims, List<Text> indepNames, int firstValue) throws IOException {

        FastTable<Integer64> elemPerDim = FastTable.newInstance();
        elemPerDim.setSize(numDims);
        elemPerDim.set(numDims - 1, Integer64.valueOf(firstValue));
        for (int i = numDims - 2; i >= 0; --i) {
            try {
                if (!tokenizer.hasMoreTokens()) {
                    //  Read in another line (the data wrapped).
                    Text aLine = readLineNoComments(reader);
                    if (aLine == null)
                        throw new IOException(format(RESOURCES.getString("EOFErrMsg"), reader.getLineNumber()));
                    tokenizer.setText(aLine);
                }

                Text token = tokenizer.nextToken().trim();
                int value = TypeFormat.parseInt(token);
                elemPerDim.set(i, Integer64.valueOf(value));

            } catch (NoSuchElementException e) {
                throw new IOException(format(RESOURCES.getString("fdaMissingNumIndepMsg"), 
                        indepNames.get(i), reader.getLineNumber()), e);
            } catch (NumberFormatException e) {
                throw new IOException(format(RESOURCES.getString("fdaParseIndepErrMsg"), 
                        indepNames.get(i), reader.getLineNumber()), e);
            }
        }

        return elemPerDim;
    }

    /**
     * Read in all the breakpoints for a single table.
     *
     * @param reader     A reader that reads in the characters from the input stream.
     * @param tokenizer  The tokenizer to use with the text set to the 1st line of
     *                   breakpoint data.
     * @param elemPerDim The number of breakpoints in each dimension to read in.
     * @param numDims    The number of dimensions to read in breakpoints for.
     * @return A list of breakpoint vectors.
     */
    private static FastTable<Float64Vector> readBreakpoints(LineNumberReader reader, TextTokenizer tokenizer,
            List<Integer64> elemPerDim, int numDims) throws IOException {

        FastTable<Float64Vector> breakpoints = FastTable.newInstance();
        FastTable<Float64> rowBP = FastTable.newInstance();

        for (int i = 0; i < numDims; ++i) {
            int dimSize = elemPerDim.get(i).intValue();
            for (int j = 0; j < dimSize; ++j) {
                if (!tokenizer.hasMoreTokens()) {
                    //  Read in another line (the data wrapped).
                    Text aLine = readLineNoComments(reader);
                    if (aLine == null)
                        throw new IOException(format(RESOURCES.getString("EOFErrMsg"), reader.getLineNumber()));
                    tokenizer.setText(aLine);
                }
                Text token = tokenizer.nextToken().trim();
                Float64 fvalue = Float64.valueOf(token);
                rowBP.add(fvalue);
            }
            breakpoints.add(Float64Vector.valueOf(rowBP));
            rowBP.reset();
        }
        FastTable.recycle(rowBP);

        return breakpoints;
    }

    /**
     * Recursively read in arrays of dependent data from an FDA formatted table file into
     * the parameter depData.
     *
     * @param dim        The dimension (independent) we are currently looping through.
     *                   Should initially be set to zero.
     * @param reader     A reader that reads in the characters from the input stream.
     * @param indep      A list of indexes into the data table (independents).
     * @param depData    The dependent data array we are reading into.
     * @param elemPerDim The number of breakpoints in each dimension of the table.
     */
    private static void rReadFDA(int dim, LineNumberReader reader, List<Integer64> indep,
            Float64NDimArray depData, List<Integer64> elemPerDim) throws IOException {
        if (dim == indep.size() - 1) {
            // Reached the highest dimension, indep[] is now ready.
            // Read in an array.
            readFDADataArray(reader, indep, depData, elemPerDim.get(dim).intValue());

        } else {

            // Loop over all the breakpoints in this dimension.
            for (int i = 0; i < elemPerDim.get(dim).intValue(); ++i) {

                // Fill in the required value for indep at this dimension.
                indep.set(dim, Integer64.valueOf(i));           // indep[dim] = i;

                // Go on to the next higher dimension.
                rReadFDA(dim + 1, reader, indep, depData, elemPerDim);
            }
        }
    }

    /**
     * Read a single array of dependent values from an FDA table file. The data read in is
     * stored in the parameter depData.
     *
     * @param reader  A reader that reads in the characters from the input stream.
     * @param indep   A list of indexes into the data table (independents).
     * @param depData The array of dependent values we are reading into.
     * @param length  The length of most rapidly varying breakpoint array (
     *                breakpoints.get(numDims-1).getDimension() ).
     */
    private static void readFDADataArray(LineNumberReader reader, List<Integer64> indep,
            Float64NDimArray depData, int length) throws IOException {

        Text aLine = readLineNoComments(reader);
        TextTokenizer tokenizer = TextTokenizer.valueOf(aLine);

        // Loop over all the breakpoints in the last independent (the array) and read in the array.
        for (int j = 0; j < length; ++j) {
            try {

                if (!tokenizer.hasMoreTokens()) {
                    //  Read in more data.
                    aLine = readLineNoComments(reader);
                    tokenizer.setText(aLine);
                }
                Text token = tokenizer.nextToken();
                double value = TypeFormat.parseDouble(token);
                indep.set(indep.size() - 1, Integer64.valueOf(j));          //  indep[indep.length - 1] = j;
                depData.set(indep, value);

            } catch (NoSuchElementException e) {
                StringBuilder msg = new StringBuilder("[");
                for (int i = 0; i < indep.size(); ++i) {
                    if (i != 0)
                        msg.append("][");
                    msg.append(indep.get(i).toString());
                }
                msg.append("]");
                throw new IOException(format(RESOURCES.getString("fdaMissingDepDataMsg"), msg.toString(), reader.getLineNumber()), e);
            } catch (NumberFormatException e) {
                StringBuilder msg = new StringBuilder("[");
                for (int i = 0; i < indep.size(); ++i) {
                    if (i != 0)
                        msg.append("][");
                    msg.append(indep.get(i).toString());
                }
                msg.append("]");
                throw new IOException(format(RESOURCES.getString("fdaParseDepDataMsg"), msg.toString(), reader.getLineNumber()), e);
            }
        }

        TextTokenizer.recycle(tokenizer);
    }

    /**
     * Reads a line of text from the supplied reader stripping out any lines that are
     * commented out.
     */
    private static Text readLineNoComments(LineNumberReader reader) throws IOException {

        String aLine = reader.readLine();
        while (aLine != null) {
            aLine = aLine.trim();
            if (aLine.equals("") || aLine.startsWith(COMMENT1) || aLine.startsWith(COMMENT2))
                aLine = reader.readLine();
            else
                break;
        }

        return (aLine != null ? Text.valueOf(aLine) : null);
    }

    /**
     * Reads lines of text from the supplied reader. If the a line starts with a comment
     * character, then the comment is added to the list of comments and the next line is
     * read in, repeating until a non-comment line is encountered. The 1st non-comment
     * line is returned by this function.
     */
    private static Text readLineKeepingComments(LineNumberReader reader, List<Text> comments) throws IOException {

        String aLine = reader.readLine();
        while (aLine != null) {
            aLine = aLine.trim();
            if (aLine.startsWith(COMMENT1) || aLine.startsWith(COMMENT2)) {
                Text comment = Text.valueOf(aLine).subtext(1).trim();
                if (comment.startsWith("*"))
                    comment = comment.subtext(1).trim();
                comments.add(comment);
                aLine = reader.readLine();

            } else if (aLine.equals(""))
                aLine = reader.readLine();

            else
                break;
        }

        return (aLine != null ? Text.valueOf(aLine) : null);
    }

    /**
     * Write out a single FDHS formatted table to an FDA file.
     *
     * @param output Reference to an output writer to write the specified table to.
     * @param table  The table to be written out.
     */
    private static void writeFDATable(BufferedWriter output, Float64Table table) throws IOException {
        final String space = "  ";

        //  Write out any table notes.
        if (!writeComments(output, table.getAllNotes())) {
            // If no table notes, write out some generic table comments.
            output.write("; ");
            table.toText().println(output);
            output.write("; Date and time written:  ");
            output.write(DateFormat.getDateTimeInstance().format(new Date()));
            output.newLine();
            output.write("; ");
            output.newLine();
        }

        // Write out the table name and independent variable names.
        Text.valueOf(table.getName()).print(output);
        output.write(space);
        List<Text> indepNames = table.getIndepNames();
        for (Text name : indepNames) {
            name.print(output);
            output.write(space);
        }

        // Write out number of breakpoints in each dimension.
        int numDims = table.getDimensions();
        for (int i = numDims - 1; i > 0; --i) {
            TypeFormat.format(table.getNumBreakpoints(i), output);
            output.write(space);
        }
        TypeFormat.format(table.getNumBreakpoints(0), output);
        output.newLine();

        // Write out the frequency card.
        output.write("all       all       ");
        output.newLine();

        // Write out all the breakpoints.
        writeBreakpoints(output, table);

        // *** Write out the dependent values.
        // Recursively write out the dependent data arrays.
        FastTable<Integer64> indep = FastTable.newInstance();
        indep.setSize(numDims);

        rWriteFDA(0, output, indep, table);

        FastTable.recycle(indep);
    }

    /**
     * Write out the vectors of breakpoints in this table.
     *
     * @param output Reference to the writer being used to output the data.
     * @param table  The table being written out.
     */
    private static void writeBreakpoints(BufferedWriter output, Float64Table table) throws IOException {

        // Loop over the breakpoint list and write out all the breakpoints.
        int numDims = table.getDimensions();
        for (int i = 0; i < numDims; ++i) {

            // Write out the breakpoints for each dimension.
            Float64Vector breakpoints = table.getBreakpoints(i);
            int length = breakpoints.getDimension();
            int count = 0;
            for (int j = 0; j < length; ++j) {
                formatNumber(NUM_DIGITS, breakpoints.getValue(j)).print(output);
                ++count;
                if (count > 4) {
                    count = 0;
                    output.newLine();
                }
            }
            if (count != 0)
                output.newLine();
        }

    }

    /**
     * Recursively write out arrays of dependent data to an FDA formatted table file.
     *
     * @param dim      The dimension (independent) we are currently looping through.
     *                 Should initially be set to zero.
     * @param output   Reference to the writer being used to output the data.
     * @param indep    An array of indexes into the data table (independents).
     * @param theTable The table we are writing out.
     */
    private static void rWriteFDA(int dim, BufferedWriter output, List<Integer64> indep,
            Float64Table theTable) throws IOException {
        if (dim == indep.size() - 1) {
            // Reached the highest dimension, indep[] is now ready.
            // Write out an array.
            writeFDADataArray(output, indep, theTable);

        } else {

            // Loop over all the breakpoints in this dimension.
            int length = theTable.getNumBreakpoints(dim);
            for (int i = 0; i < length; ++i) {

                // Fill in the required value for indep[] at this dimension.
                indep.set(dim, Integer64.valueOf(i));           //  indep[dim] = i;

                // Go on to the next higher dimension.
                rWriteFDA(dim + 1, output, indep, theTable);
            }
        }
    }

    /**
     * Write a single array of dependent values to an FDA table file.
     *
     * @param output   A writer used to output the data.
     * @param indep    An array of indexes into the data table (independents).
     * @param theTable The table we are writing out.
     */
    private static void writeFDADataArray(BufferedWriter output, List<Integer64> indep,
            Float64Table theTable) throws IOException {
        int count = 0;
        int numDims = indep.size();
        int length = theTable.getNumBreakpoints(numDims - 1);

        // Loop over all the breakpoints in the last independent (the array)
        // and write out the data array.
        for (int j = 0; j < length; ++j) {
            indep.set(numDims - 1, Integer64.valueOf(j));           //  indep[numDims - 1] = j;
            double value = theTable.getValue(indep);
            formatNumber(NUM_DIGITS, value).print(output);
            ++count;
            if (count > 4) {
                count = 0;
                output.newLine();
            }
        }
        if (count != 0)
            output.newLine();
    }

    /**
     * Format a number for output to a table file. This method formats the given number
     * using the a TextBuilder It then adds spaces to the start of the formatted number
     * until the string reaches the specified length.
     *
     * @param size   The overall length of the formatted number including the decimal
     *               point, minus sign, "E" notation, etc.
     * @param number The number to be formatted.
     */
    private static Text formatNumber(int size, double number) throws IOException {

        TextBuilder buffer = TextBuilder.newInstance();
        TypeFormat.format(number, 8, false, false, buffer);
        int length = buffer.length();
        if (length > size) {
            buffer.clear();
            TypeFormat.format(number, 7, true, false, buffer);
            length = buffer.length();
        }
        while (length < size) {
            buffer.insert(0, SPACE);
            ++length;
        }
        Text text = buffer.toText();
        TextBuilder.recycle(buffer);

        return text;
    }

    /**
     * Write out any database notes as comments to the FDA file.
     *
     * @param output A writer used to output the data.
     * @param notes  The notes to be written out.
     * @return <code>true</code> if any comment were written, <code>false</code>
     *         otherwise.
     */
    private static boolean writeComments(BufferedWriter output, List<Text> notes)
            throws IOException {
        boolean retVal = false;
        for (Text note : notes) {
            output.write("; ");
            note.println(output);
            retVal = true;
        }

        return retVal;
    }

}
