/*
 *   FReaderFactory  -- A factory object that returns FTableReader objects.
 *
 *   Copyright (C) 2003-2024, by Joseph A. Huwaldt.
 *   All rights reserved.
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.tables;

import java.io.*;
import java.net.URL;
import java.text.MessageFormat;
import static java.text.MessageFormat.format;
import java.util.Arrays;
import java.util.Enumeration;
import static java.util.Objects.requireNonNull;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import javolution.util.FastComparator;
import javolution.util.FastSet;
import javolution.util.FastTable;

/**
 * This class returns a specific {@link FTableReader} object that can read in the
 * specified file. This class implements a pluggable architecture. A new
 * <code>FTableReader</code> class can be added by simply creating a subclass of
 * <code>FTableReader</code>, creating a <code>FTableReader.properties</code> file that
 * refers to it, and putting that properties file somewhere in the Java class path.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: April 3, 2000
 * @version January 15, 2024
 */
public final class FTReaderFactory {

    private static final ResourceBundle RESOURCES = FTableReader.RESOURCES;
    
    /**
     * All class loader resources with the name <code>FTableReader.properties</code> are
     * loaded as .properties definitions and merged together to create a global list of
     * reader mappings.
     */
    private static final String MAPPING_RES_NAME = "FTableReader.properties";

    //  An array containing a reference to all the readers that have been found.
    private static final FTableReader[] _allReaders; // set in <clinit>

    //  Locate and load all the readers we can find.
    static {
        FTableReader[] temp = null;
        try {
            temp = loadResourceList(MAPPING_RES_NAME, getClassLoader());

        } catch (Exception e) {
            String msg = format(RESOURCES.getString("factoryCouldNotLoadMsg"), MAPPING_RES_NAME);
            Logger.getLogger(FTReaderFactory.class.getName()).log(Level.SEVERE, msg, e);
        }

        _allReaders = temp;
    }

    // this class is not extendible
    private FTReaderFactory() { }

    /**
     * Method that attempts to find a {@link FTableReader} object that might be able to
     * read the data in the specified input stream.
     *
     * @param inputStream An input stream to the file that a reader is to be returned for.
     * @param pathName    The path or file name for the file that a reader is to be
     *                    returned for.
     * @return A reader that is appropriate for the specified file, or <code>null</code>
     *         if the user cancels the multiple reader selection dialog.
     * @throws IOException if an appropriate reader for the file could not be found.
     */
    public static FTableReader getReader(InputStream inputStream, String pathName) throws IOException {

        //  Get the list of data readers that are available.
        FTableReader[] allReaders = getAllReaders();
        if (allReaders == null || allReaders.length < 1)
            throw new IOException(RESOURCES.getString("factoryNoReadersAvailableMsg"));

        FastTable<FTableReader> list = FastTable.newInstance();
        BufferedInputStream input;

        try {
            //  Wrap input stream in a buffered stream.
            input = new BufferedInputStream(inputStream);
            String name = new File(pathName).getName();

            //  Loop through all the available readers and see if any of them will work.
            for (FTableReader reader : allReaders) {

                //  Mark the current position in the input stream (the start of the stream).
                input.mark(1024000);        //  1MB

                //  Can this reader read the specified input stream?
                int canReadFile = reader.canReadData(name, input);

                //  If the reader is certain it can read the data, use it.
                if (canReadFile == FTableReader.YES) {
                    FastTable.recycle(list);
                    return reader;
                }

                //  Otherwise, build a list of maybes.
                if (canReadFile == FTableReader.MAYBE)
                    list.add(reader);

                //  Return to the start of the stream for the next pass.
                input.reset();
            }

        } catch (Exception e) {
            FastTable.recycle(list);
            throw new IOException(RESOURCES.getString("factoryFileTypeDetErr"), e);
        }

        if (list.size() < 1) {
            FastTable.recycle(list);
            throw new IOException(RESOURCES.getString("factoryCantDetTypeErr"));

        }
        //  If there is only one reader in the list, try and use it.
        FTableReader selectedReader;
        if (list.size() == 1)
            selectedReader = list.get(0);

        else {
            //  Ask the user to select which reader they want to try and use.
            FTableReader[] possibleValues = list.toArray(new FTableReader[list.size()]);
            selectedReader = (FTableReader)JOptionPane.showInputDialog(null,
                    format(RESOURCES.getString("factoryChooseFormatMsg"), pathName), 
                    RESOURCES.getString("factoryChooseFormatTitle"),
                    JOptionPane.INFORMATION_MESSAGE, null, possibleValues, possibleValues[0]);
        }

        //  Clean up before leaving.
        FastTable.recycle(list);

        return selectedReader;
    }

    /**
     * Method that attempts to find an {@link FTableReader} object that might be able to
     * read the data in the specified file.
     *
     * @param theFile The file to find a reader for.
     * @return A reader that is appropriate for the specified file, or <code>null</code>
     *         if the user cancels the multiple reader selection dialog.
     * @throws IOException if an appropriate reader for the file could not be found.
     */
    public static FTableReader getReader(File theFile) throws IOException {

        if (!theFile.exists())
            throw new IOException(format(RESOURCES.getString("factoryCantFindFileMsg"), theFile.getName()));

        FTableReader reader;
        try (FileInputStream input = new FileInputStream(theFile)) {
            String name = theFile.getName();

            reader = getReader(input, name);
        }

        return reader;
    }

    /**
     * Method that returns a list of all the {@link FTableReader} objects found by this
     * factory.
     *
     * @return An array of <code>FTableReader</code> objects (can be null if static init
     *         failed).
     */
    public static FTableReader[] getAllReaders() {
        return _allReaders;
    }

    /*
     * Loads a reader list that is a union of *all* resources named
     * 'resourceName' as seen by 'loader'. Null 'loader' is equivalent to the
     * application loader.
     */
    private static FTableReader[] loadResourceList(final String resourceName, ClassLoader loader) {
        if (loader == null)
            loader = ClassLoader.getSystemClassLoader();

        final FastSet<FTableReader> result = FastSet.newInstance();

        try {
            // NOTE: using getResources() here
            final Enumeration<URL> resources = loader.getResources(resourceName);

            if (resources != null) {
                // merge all mappings in 'resources':

                while (resources.hasMoreElements()) {
                    final URL url = resources.nextElement();
                    final Properties mapping;

                    InputStream urlIn = null;
                    try {
                        urlIn = url.openStream();

                        mapping = new Properties();
                        mapping.load(urlIn); // load in .properties format

                    } catch (IOException ioe) {
                        // ignore this resource and go to the next one
                        continue;

                    } finally {
                        if (urlIn != null)
                            try {
                                urlIn.close();
                            } catch (Exception ignore) { }
                    }

                    // load all readers specified in 'mapping':
                    for (Enumeration<?> keys = mapping.propertyNames(); keys.hasMoreElements();) {
                        final String format = (String)keys.nextElement();
                        final String implClassName = mapping.getProperty(format);

                        result.add(loadResource(implClassName, loader));
                    }
                }
            }

        } catch (IOException ignore) {
            // ignore: an empty result will be returned
        }

        //  Convert result Set to an array.
        FTableReader[] resultArr = result.toArray(new FTableReader[result.size()]);

        //  Sort the array using the specified comparator.
        Arrays.sort(resultArr, FastComparator.DEFAULT);

        //  Clean up before leaving.
        FastSet.recycle(result);

        //  Output the sorted array.
        return resultArr;
    }

    /*
     * Loads and initializes a single resource for a given format name via a
     * given class loader. For simplicity, all errors are converted to
     * RuntimeExceptions.
     */
    private static FTableReader loadResource(final String className, final ClassLoader loader) {
        requireNonNull(className, MessageFormat.format(RESOURCES.getString("paramNullErr"), "className"));
        requireNonNull(loader, MessageFormat.format(RESOURCES.getString("paramNullErr"), "loader"));

        final Class<?> cls;
        final Object reader;
        try {
            cls = Class.forName(className, true, loader);
            reader = cls.getDeclaredConstructor().newInstance();

        } catch (Exception e) {
            throw new RuntimeException(format(RESOURCES.getString("factoryCouldNotLoadMsg"), className)
                    + ": " + e.getMessage(), e);
        }

        if (!(reader instanceof FTableReader))
            throw new RuntimeException(format(RESOURCES.getString("factoryWrongClassMsg"), cls.getName()));

        return (FTableReader)reader;
    }

    /**
     * This method decides on which class loader is to be used by all resource/class
     * loading in this class. At the very least you should use the current thread's
     * context loader. A better strategy would be to use techniques shown in
     * http://www.javaworld.com/javaworld/javaqa/2003-06/01-qa-0606-load.html
     */
    private static ClassLoader getClassLoader() {
        return Thread.currentThread().getContextClassLoader();
    }

}
