/*
 * Float64NDimArray -- An array of Float64 values with an arbitrary number of dimension.
 *
 * Copyright (C) 1999-2024, by Joseph A. Huwaldt.
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *   
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Or visit:  http://www.gnu.org/licenses/lgpl.html
 */
package jahuwaldt.js.tables;

import java.text.MessageFormat;
import static java.text.MessageFormat.format;
import java.util.Arrays;
import java.util.List;
import static java.util.Objects.isNull;
import static java.util.Objects.requireNonNull;
import java.util.ResourceBundle;
import javolution.context.ArrayFactory;
import javolution.context.ObjectFactory;
import javolution.lang.MathLib;
import javolution.lang.Realtime;
import javolution.text.Text;
import javolution.text.TextBuilder;
import javolution.util.FastTable;
import javolution.xml.XMLFormat;
import javolution.xml.XMLSerializable;
import javolution.xml.stream.XMLStreamException;
import org.jscience.mathematics.number.Float64;
import org.jscience.mathematics.number.Integer64;
import org.jscience.mathematics.vector.Float64Vector;

/**
 * An array of <code>Float64</code> values that can have an arbitrary number of
 * dimensions. It could be 1D, 2D, 3D,..., nD.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt, Date: July 15, 1999
 * @version January 15, 2024
 */
public final class Float64NDimArray implements Realtime, XMLSerializable {

    private static final long serialVersionUID = 8944471868568052320L;

    private static final ResourceBundle RESOURCES = Float64Table.RESOURCES;

    /**
     * The number of elements per dimension.
     */
    private FastTable<Integer64> _sizes;

    /**
     * The number of data elements stored in this nD array.
     */
    private int _numElements;

    /**
     * The 1D array that contains all the data strung out end-to-end.
     */
    private double[] _data;

    //-----------------------------------------------------------------------------
    /**
     * Returns a <code>Float64NDimArray</code> object with all the array elements set to
     * zero.
     *
     * @param dimensionSizes List of <code>Integer64</code> values indicating the number
     *                       of elements in each dimension of the array.
     * @return A Float64NDimArray object with all the elements set to zero.
     */
    public static Float64NDimArray newInstance(List<Integer64> dimensionSizes) {
        requireNonNull(dimensionSizes, MessageFormat.format(RESOURCES.getString("paramNullErr"), "dimensionSizes"));
        
        // Determine how many elements are in this array.
        int numDims = dimensionSizes.size();
        int numElements = 1;
        for (int i = 0; i < numDims; ++i)
            numElements *= dimensionSizes.get(i).intValue();

        //  Get an nD array instance and store the dimension sizes.
        Float64NDimArray arr = Float64NDimArray.newInstance(numElements);
        arr._sizes.addAll(dimensionSizes);

        // Clear the stored data array (in case it was recycled).
        arr.setAll(0);

        return arr;
    }

    /**
     * Returns a <code>Float64NDimArray</code> object with the specified dimensions with
     * the data set by the specified 1D array of vectorized (flattened) array data.
     *
     * @param dimensionSizes List of <code>Integer64</code> values indicating the number
     *                       of elements in each dimension of the array.
     * @param vector         The data for this n-dimensional array vectorized (flattened)
     *                       into a 1D vector. The number of elements in this array must
     *                       equal the number indicated by the dimensionSizes.
     * @return A Float64NDimArray set to the specified 1D array of flattened array data.
     */
    public static Float64NDimArray valueOf(List<Integer64> dimensionSizes, Float64Vector vector) {
        requireNonNull(dimensionSizes, MessageFormat.format(RESOURCES.getString("paramNullErr"), "dimensionSizes"));
        requireNonNull(vector, MessageFormat.format(RESOURCES.getString("paramNullErr"), "vector"));
        
        // Determine how many elements are in this array.
        int numDims = dimensionSizes.size();
        int numElements = 1;
        for (int i = 0; i < numDims; ++i)
            numElements *= dimensionSizes.get(i).intValue();

        //  Does the input array have the correct number of elements?
        if (numElements != vector.getDimension())
            throw new IllegalArgumentException(RESOURCES.getString("f64VectIncorNumElementsMsg"));

        //  Get an nD array instance and store the dimension sizes.
        Float64NDimArray arr = Float64NDimArray.newInstance(numElements);
        arr._sizes.addAll(dimensionSizes);

        // Copy in the vector of data.
        for (int i = 0; i < numElements; ++i)
            arr._data[i] = vector.getValue(i);

        return arr;
    }

    /**
     * Returns a <code>Float64NDimArray</code> object containing the values stored in a
     * given 1D vector.
     *
     * @param vector A 1D vector to be converted into a <code>Float64NDimArray</code>
     *               object.
     * @return A Float64NDimArray object with the values from the specified vector.
     */
    public static Float64NDimArray valueOf(Float64Vector vector) {
        requireNonNull(vector, MessageFormat.format(RESOURCES.getString("paramNullErr"), "vector"));
        
        // Determine how many elements are in this array.
        int numElements = vector.getDimension();

        //  Get an nD array instance and store the dimension sizes.
        Float64NDimArray arr = Float64NDimArray.newInstance(numElements);
        arr._sizes.add(Integer64.valueOf(numElements));

        // Copy in the array of data.
        for (int i = 0; i < numElements; ++i)
            arr._data[i] = vector.getValue(i);

        return arr;
    }

    /**
     * Returns a <code>Float64NDimArray</code> object containing the values stored in a
     * given 1D array.
     *
     * @param srcArray A 1D array to be converted into a <code>Float64NDimArray</code>
     *                 object.
     * @return A Float64NDimArray with the values from the specified array.
     */
    public static Float64NDimArray valueOf(double... srcArray) {
        requireNonNull(srcArray, MessageFormat.format(RESOURCES.getString("paramNullErr"), "srcArray"));
        
        // Determine how many elements are in this array.
        int numElements = srcArray.length;

        //  Get an nD array instance and store the dimension sizes.
        Float64NDimArray arr = Float64NDimArray.newInstance(numElements);
        arr._sizes.add(Integer64.valueOf(numElements));

        // Copy in the array of data.
        System.arraycopy(srcArray, 0, arr._data, 0, numElements);

        return arr;
    }

    /**
     * Returns a <code>Float64NDimArray</code> object containing the values stored in a
     * given 2D array.
     *
     * @param srcArray A list of vectors (a 2D matrix) to be converted into a
     *                 <code>Float64NDimArray</code> object. The input data must be
     *                 "rectangular". This means that each row of the list must have the
     *                 same number of columns in it.
     * @return A Float64NDimArray with the values from the specified array.
     */
    public static Float64NDimArray valueOf(List<Float64Vector> srcArray) {
        requireNonNull(srcArray, MessageFormat.format(RESOURCES.getString("paramNullErr"), "srcArray"));
        
        // Determine how many elements are in this array.
        int length1 = srcArray.size();
        int length2 = srcArray.get(0).getDimension();
        int numElements = length1 * length2;

        //  Get an nD array instance and store the dimension sizes.
        Float64NDimArray arr = Float64NDimArray.newInstance(numElements);
        arr._sizes.add(Integer64.valueOf(length1));
        arr._sizes.add(Integer64.valueOf(length2));

        // Copy in the array of data.
        int pos = 0;
        for (Float64Vector vector : srcArray) {
            for (int i = 0; i < length2; ++i, ++pos)
                arr._data[pos] = vector.getValue(i);
        }

        return arr;
    }

    /**
     * Returns a <code>Float64NDimArray</code> object containing the values stored in a
     * given 2D array.
     *
     * @param srcArray A 2D array to be converted into a <code>Float64NDimArray</code>
     *                 object. The input array must be "rectangular". This means that each
     *                 row of the array must have the same number of columns in it.
     * @return A Float64NDimArray with the values from the specified array.
     */
    public static Float64NDimArray valueOf(double[][] srcArray) {
        requireNonNull(srcArray, MessageFormat.format(RESOURCES.getString("paramNullErr"), "srcArray"));
        
        // Determine how many elements are in this array.
        int length1 = srcArray.length;
        int length2 = srcArray[0].length;
        int numElements = length1 * length2;

        //  Get an nD array instance and store the dimension sizes.
        Float64NDimArray arr = Float64NDimArray.newInstance(numElements);
        arr._sizes.add(Integer64.valueOf(length1));
        arr._sizes.add(Integer64.valueOf(length2));

        // Copy in the array of data.
        int pos = 0;
        for (int i = 0; i < length1; ++i) {
            System.arraycopy(srcArray[i], 0, arr._data, pos, length2);
            pos += length2;
        }

        return arr;
    }

    /**
     * Returns a <code>Float64NDimArray</code> object containing the values stored in a
     * given 3D array.
     *
     * @param srcArray A 3D array (list of lists of vectors) to be converted into a
     *                 <code>Float64NDimArray</code> object. The input array must be
     *                 "rectangular". This means that every every row of every rank must
     *                 have the same number of columns.
     * @return A Float64NDimArray object with the values from the specified array.
     */
    public static Float64NDimArray valueOf3D(List<List<Float64Vector>> srcArray) {
        requireNonNull(srcArray, MessageFormat.format(RESOURCES.getString("paramNullErr"), "srcArray"));
        
        // Determine how many elements are in this array.
        int length1 = srcArray.size();
        int length2 = srcArray.get(0).size();
        int length3 = srcArray.get(0).get(0).getDimension();
        int numElements = length1 * length2 * length3;

        //  Get an nD array instance and store the dimension sizes.
        Float64NDimArray arr = Float64NDimArray.newInstance(numElements);
        arr._sizes.add(Integer64.valueOf(length1));
        arr._sizes.add(Integer64.valueOf(length2));
        arr._sizes.add(Integer64.valueOf(length3));

        // Copy in the array of data.
        int pos = 0;
        for (List<Float64Vector> list : srcArray) {
            for (Float64Vector vector : list) {
                for (int i = 0; i < length3; ++i, ++pos)
                    arr._data[pos] = vector.getValue(i);
            }
        }

        return arr;
    }

    /**
     * Returns a <code>Float64NDimArray</code> object containing the values stored in a
     * given 3D array.
     *
     * @param srcArray A 3D array to be converted into a <code>Float64NDimArray</code>
     *                 object. The input array must be "rectangular". This means that
     *                 every every row of every rank must have the same number of columns.
     * @return A Float64NDimArray with values from the specified array.
     */
    public static Float64NDimArray valueOf(double[][][] srcArray) {
        requireNonNull(srcArray, MessageFormat.format(RESOURCES.getString("paramNullErr"), "srcArray"));
        
        // Determine how many elements are in this array.
        int length1 = srcArray.length;
        int length2 = srcArray[0].length;
        int length3 = srcArray[0][0].length;
        int numElements = length1 * length2 * length3;

        //  Get an nD array instance and store the dimension sizes.
        Float64NDimArray arr = Float64NDimArray.newInstance(numElements);
        arr._sizes.add(Integer64.valueOf(length1));
        arr._sizes.add(Integer64.valueOf(length2));
        arr._sizes.add(Integer64.valueOf(length3));

        // Copy in the array of data.
        int pos = 0;
        for (int i = 0; i < length1; ++i) {
            for (int j = 0; j < length2; ++j) {
                System.arraycopy(srcArray[i][j], 0, arr._data, pos, length3);
                pos += length3;
            }
        }

        return arr;
    }

    //-----------------------------------------------------------------------------
    /**
     * Set the value of a particular element in the n dimensional array.
     *
     * @param position A list of indexes into the array of the value to be set. For
     *                 example, for a 3D array the position could be { 0, 3, 2 } which
     *                 would translate to [0][3][2] in a standard Java array.
     * @param value    The value to be stored in the array.
     * @throws ArrayIndexOutOfBoundsException if the input position exceeds the array
     * dimensions.
     */
    public void set(List<Integer64> position, Float64 value) {
        requireNonNull(position, MessageFormat.format(RESOURCES.getString("paramNullErr"), "position"));
        requireNonNull(value, MessageFormat.format(RESOURCES.getString("paramNullErr"), "value"));
        
        int pos = getOffset(position);
        _data[pos] = value.doubleValue();
    }

    /**
     * Set the value of a particular element in the n dimensional array.
     *
     * @param position A list of indexes into the array of the value to be set. For
     *                 example, for a 3D array the position could be { 0, 3, 2 } which
     *                 would translate to [0][3][2] in a standard Java array.
     * @param value    The value to be stored in the array.
     * @throws ArrayIndexOutOfBoundsException if the input position exceeds the array
     * dimensions.
     */
    public void set(List<Integer64> position, double value) {
        int pos = getOffset(requireNonNull(position, MessageFormat.format(RESOURCES.getString("paramNullErr"), "position")));
        _data[pos] = value;
    }

    /**
     * Set the value of a particular element in the 1-dimensional array. This method works
     * only if the array contained by this object is one-dimensional. This convenience
     * method is provided for increased performance when you know you have a 1-D array.
     *
     * @param position The index into the 1-D array of the value to be set.
     * @param value    The value to be stored in the array.
     * @throws ArrayIndexOutOfBoundsException if this array is not 1-dimensional.
     */
    public void set(int position, Float64 value) {
        requireNonNull(value, MessageFormat.format(RESOURCES.getString("paramNullErr"), "value"));
        
        if (_sizes.size() != 1)
            throw new ArrayIndexOutOfBoundsException(RESOURCES.getString("f64WrongDimensionsErr"));
        if (position >= _numElements)
            throw new ArrayIndexOutOfBoundsException(
                    format(RESOURCES.getString("f64ArrayIndexOutOfBoundsErr"), position, _numElements));
        _data[position] = value.doubleValue();
    }

    /**
     * Set the value of a particular element in the 1-dimensional array. This method works
     * only if the array contained by this object is one-dimensional. This convenience
     * method is provided for increased performance when you know you have a 1-D array.
     *
     * @param position The index into the 1-D array of the value to be set.
     * @param value    The value to be stored in the array.
     * @throws ArrayIndexOutOfBoundsException if this array is not 1-dimensional.
     */
    public void set(int position, double value) {
        if (_sizes.size() != 1)
            throw new ArrayIndexOutOfBoundsException(RESOURCES.getString("f64WrongDimensionsErr"));
        if (position >= _numElements)
            throw new ArrayIndexOutOfBoundsException(
                    format(RESOURCES.getString("f64ArrayIndexOutOfBoundsErr"), position, _numElements));

        _data[position] = value;
    }

    /**
     * Get the value of a particular element in the n dimensional array.
     *
     * @param position A list of indexes into the array of the value to be returned. For
     *                 example, for a 3D array the position could be { 0, 3, 2 } which
     *                 would translate to [0][3][2] in a standard Java array.
     * @return Returns the value at the location in the array given by position.
     * @throws ArrayIndexOutOfBoundsException if the input position exceeds the array
     * dimensions.
     */
    public Float64 get(List<Integer64> position) {
        int pos = getOffset(requireNonNull(position, MessageFormat.format(RESOURCES.getString("paramNullErr"), "position")));
        return Float64.valueOf(_data[pos]);
    }

    /**
     * Get the value of a particular element in the n dimensional array as a
     * <code>double</code>.
     *
     * @param position A list of indexes into the array of the value to be returned. For
     *                 example, for a 3D array the position could be { 0, 3, 2 } which
     *                 would translate to [0][3][2] in a standard Java array.
     * @return Returns the value at the location in the array given by position as a
     *         <code>double</code>.
     * @throws ArrayIndexOutOfBoundsException if the input position exceeds the array
     * dimensions.
     */
    public double getValue(List<Integer64> position) {
        int pos = getOffset(requireNonNull(position, MessageFormat.format(RESOURCES.getString("paramNullErr"), "position")));
        return _data[pos];
    }

    /**
     * Get the value of a particular element in the 1-dimensional array. This method works
     * only if the array contained by this object is one-dimensional. This convenience
     * method is provided for increased performance when you know you have a 1-D array.
     *
     * @param position The index into the 1D array of the value to be returned.
     * @return Returns the value at the location in the array given by position.
     * @throws ArrayIndexOutOfBoundsException if this array is not 1-dimensional.
     */
    public Float64 get(int position) {
        if (_sizes.size() != 1)
            throw new ArrayIndexOutOfBoundsException(RESOURCES.getString("f64WrongDimensionsErr"));
        if (position >= _numElements)
            throw new ArrayIndexOutOfBoundsException(
                    format(RESOURCES.getString("f64ArrayIndexOutOfBoundsErr"), position, _numElements));

        return Float64.valueOf(_data[position]);
    }

    /**
     * Get the value of a particular element in the 1-dimensional array as a
     * <code>double</code>. This method works only if the array contained by this object
     * is one-dimensional. This convenience method is provided for increased performance
     * when you know you have a 1-D array.
     *
     * @param position The index into the 1D array of the value to be returned.
     * @return Returns the value at the location in the array given by position as a
     *         <code>double</code>.
     * @throws ArrayIndexOutOfBoundsException if this array is not 1-dimensional.
     */
    public double getValue(int position) {
        if (_sizes.size() != 1)
            throw new ArrayIndexOutOfBoundsException(RESOURCES.getString("f64WrongDimensionsErr"));
        if (position >= _numElements)
            throw new ArrayIndexOutOfBoundsException(
                    format(RESOURCES.getString("f64ArrayIndexOutOfBoundsErr"), position, _numElements));

        return _data[position];
    }

    /**
     * Returns the vectorization of this nD array. The vectorization is a 1-D column
     * vector made up of all the data in the nD array (rows, columns, ranks, etc) strung
     * out end-to-end.
     *
     * @return the vectorization of this array.
     */
    public Float64Vector vectorization() {
        FastTable<Float64> table = FastTable.newInstance();
        int numElements = _numElements;
        for (int i = 0; i < numElements; ++i)
            table.add(Float64.valueOf(_data[i]));
        Float64Vector vector = Float64Vector.valueOf(table);
        FastTable.recycle(table);
        return vector;
    }

    /**
     * Set all the values stored in this array to the given value.
     *
     * @param value Value to place in every element of the array.
     */
    public void setAll(Float64 value) {
        requireNonNull(value, MessageFormat.format(RESOURCES.getString("paramNullErr"), "value"));
        double dvalue = value.doubleValue();
        Arrays.fill(_data, dvalue);
    }

    /**
     * Set all the values stored in this array to the given value.
     *
     * @param value Value to place in every element of the array.
     */
    public void setAll(double value) {
        Arrays.fill(_data, value);
    }

    /**
     * Returns the number of dimensions in this n-dimensional array.
     *
     * @return Returns the number of dimensions in this array object.
     */
    public int getDimensions() {
        return _sizes.size();
    }

    /**
     * Returns the length of a given dimension of the array.
     *
     * @param dimension The dimension to return the length of.
     * @return Returns the number of elements in a given dimensional direction (the size
     *         of that dimension).
     */
    public int length(int dimension) {
        return _sizes.get(dimension).intValue();
    }

    /**
     * Returns the overall number of elements in the array.
     *
     * @return Returns the overall number of elements in the array.
     */
    public int size() {
        return _numElements;
    }

    /**
     * Copy a given Java array into a Float64NDimArray array.
     *
     * @param srcArray The source array to copy from.
     * @param srcPos   Position in the source array to start copying from.
     * @param dstArray The destination Float64NDimArray to copy into.
     * @param dstPos   A list of indexes in the nDim array to copy to.
     * @param length   The number of array elements to copy.
     * @throws ArrayIndexOutOfBoundsException if the specified position and length will
     * not fit into the dstArray.
     */
    public static void arraycopy(double[] srcArray, int srcPos,
            Float64NDimArray dstArray, List<Integer64> dstPos, int length) {
        requireNonNull(srcArray, MessageFormat.format(RESOURCES.getString("paramNullErr"), "srcArray"));
        int dPos = dstArray.getOffset(requireNonNull(dstPos, MessageFormat.format(RESOURCES.getString("paramNullErr"), "dstPos")));
        int nDim = dstArray._sizes.size() - 1;

        if (dstPos.get(nDim).intValue() + length > dstArray._sizes.get(nDim).intValue())
            throw new ArrayIndexOutOfBoundsException();

        System.arraycopy(srcArray, srcPos, dstArray._data, dPos, length);
    }

    /**
     * Copy a given Float64NDimArray array into a Java array.
     *
     * @param srcArray The source Float64NDimArray array to copy from.
     * @param srcPos   A list of indexes in the source array to start copying from.
     * @param dstArray The destination Java array to copy into.
     * @param dstPos   Position in the destination array to copy to.
     * @param length   The number of array elements to copy.
     * @throws ArrayIndexOutOfBoundsException if the specified position and length will
     * not fit into the dstArray.
     */
    public static void arraycopy(Float64NDimArray srcArray, List<Integer64> srcPos,
            double[] dstArray, int dstPos, int length) {
        requireNonNull(dstArray, MessageFormat.format(RESOURCES.getString("paramNullErr"), "dstArray"));
        int sPos = srcArray.getOffset(requireNonNull(srcPos, MessageFormat.format(RESOURCES.getString("paramNullErr"), "srcPos")));
        int nDim = srcArray._sizes.size() - 1;

        if (srcPos.get(nDim).intValue() + length > srcArray._sizes.get(nDim).intValue())
            throw new ArrayIndexOutOfBoundsException();

        System.arraycopy(srcArray._data, sPos, dstArray, dstPos, length);
    }

    /**
     * Returns the text representation of this n-D array.
     *
     * @return the text representation of this n-D array.
     */
    @Override
    public Text toText() {
        final int numDims = this.getDimensions();
        final int size = this.size();
        TextBuilder tmp = TextBuilder.newInstance();
        tmp.append("{ dimensions = {");

        for (int i = 0; i < numDims; i++) {
            tmp.append(_sizes.get(i));
            if (i != numDims - 1) {
                tmp.append(", ");
            }
        }
        tmp.append("}, data = {");
        for (int i = 0; i < size; i++) {
            tmp.append(_data[i]);
            if (i != size - 1) {
                tmp.append(", ");
            }
        }
        tmp.append('}');
        Text txt = tmp.toText();
        TextBuilder.recycle(tmp);
        return txt;
    }

    /**
     * Returns the text representation of this nD array as a
     * <code>java.lang.String</code>.
     *
     * @return <code>toText().toString()</code>
     */
    @Override
    public final String toString() {
        return toText().toString();
    }

    /**
     * Compares this <code>Float64NDimArray</code> against the specified object for strict
     * equality (same dimensions and same values).
     *
     * @param obj the object to compare with.
     * @return <code>true</code> if this nD array is identical to that nD array;
     *         <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if ((obj == null) || (obj.getClass() != this.getClass()))
            return false;

        Float64NDimArray that = (Float64NDimArray)obj;
        if (this._numElements != that._numElements)
            return false;
        if (!this._sizes.equals(that._sizes))
            return false;

        int numElements = this._numElements;
        for (int i = 0; i < numElements; ++i)
            if (this._data[i] != that._data[i])
                return false;

        return true;
    }

    /**
     * Returns the hash code for this <code>Float64NDimArray</code>.
     *
     * @return the hash code value.
     */
    @Override
    public int hashCode() {
        int hash = 7;

        hash = hash * 31 + _sizes.hashCode();

        Float64Vector vector = vectorization();
        int var_code = vector.hashCode();
        hash = hash * 31 + var_code;

        return hash;
    }

    /**
     * Returns a copy of this nD array
     * {@link javolution.context.AllocatorContext allocated} by the calling thread
     * (possibly on the stack).
     *
     * @return an identical and independent copy of this array.
     */
    public Float64NDimArray copy() {
        return copyOf(this);
    }

    /**
     * Recycles a <code>Float64NDimArray</code> instance immediately (on the stack when
     * executing in a <code>StackContext</code>).
     *
     * @param instance The instance to be recycled.
     */
    public static void recycle(Float64NDimArray instance) {
        FACTORY.recycle(requireNonNull(instance, MessageFormat.format(RESOURCES.getString("paramNullErr"), "instance")));
    }

    /**
     * Returns the offset into the 1D data array of the element indicated by the index
     * into the nD array.
     *
     * @param position List of indexes into the nD array.
     * @return Returns the position in the data array of the indicated nD position.
     * @throws ArrayIndexOutOfBoundsException if the input position exceeds the array
     * dimensions.
     */
    private int getOffset(List<Integer64> position) {
        int numDims = _sizes.size();

        if (isNull(position) || position.size() != numDims)
            throw new ArrayIndexOutOfBoundsException(RESOURCES.getString("f64WrongDimensionsErr"));

        int offset = 0;
        int factor = 1;
        for (int i = numDims - 1; i >= 0; --i) {
            int pos = position.get(i).intValue();
            int size = _sizes.get(i).intValue();

            if (pos >= size || pos < 0)
                throw new ArrayIndexOutOfBoundsException();

            offset += pos * factor;
            factor *= size;
        }

        return offset;
    }

    //-----------------------------------------------------------------------------
    /**
     * Holds the default XML representation. For example:
     * <pre>
     *    &lt;Float64NDimArray&gt;
     *        &lt;Dimensions&gt;
     *            &lt;Integer64 value="3" /&gt;
     *            &lt;Integer64 value="3" /&gt;
     *        &lt;/Dimensions&gt;
     *        &lt;Data dimension="9"&gt;
     *            &lt;Float64 value="1.0" /&gt;
     *            &lt;Float64 value="0.0" /&gt;
     *            &lt;Float64 value="0.0" /&gt;
     *            &lt;Float64 value="0.0" /&gt;
     *            &lt;Float64 value="1.0" /&gt;
     *            &lt;Float64 value="0.0" /&gt;
     *            &lt;Float64 value="0.0" /&gt;
     *            &lt;Float64 value="0.0" /&gt;
     *            &lt;Float64 value="1.0" /&gt;
     *        &lt;/Data&gt;
     *    &lt;/Float64NDimArray&gt;
     * </pre>
     */
    protected static final XMLFormat<Float64NDimArray> XML = new XMLFormat<Float64NDimArray>(Float64NDimArray.class) {

        @SuppressWarnings("unchecked")
        @Override
        public Float64NDimArray newInstance(Class<Float64NDimArray> cls, InputElement xml) throws XMLStreamException {

            //  Read the data from the XML element.
            FastTable<Integer64> sizesTable = xml.get("Dimensions", FastTable.class);
            Float64Vector dataVector = xml.get("Data", Float64Vector.class);

            if (xml.hasNext()) {
                FastTable.recycle(sizesTable);
                throw new XMLStreamException("Too many elements");
            }

            //  Create the nD array.
            Float64NDimArray arr = null;
            try {
                arr = Float64NDimArray.valueOf(sizesTable, dataVector);

            } catch (IllegalArgumentException e) {
                throw new XMLStreamException("Number of elements in Data inconsistant with DimensionSizes.", e);
            }

            return arr;
        }

        @Override
        public void read(InputElement xml, Float64NDimArray arr) throws XMLStreamException {
            //  Nothing to do.
        }

        @Override
        public void write(Float64NDimArray A, OutputElement xml) throws XMLStreamException {

            //  Box up the data vector for output.
            Float64Vector dataVector = A.vectorization();

            xml.add(A._sizes, "Dimensions", FastTable.class);
            xml.add(dataVector, "Data", Float64Vector.class);

        }
    };

    ///////////////////////
    // Factory creation. //
    ///////////////////////
    private static final ObjectFactory<Float64NDimArray> FACTORY = new ObjectFactory<Float64NDimArray>() {
        @Override
        protected Float64NDimArray create() {
            return new Float64NDimArray();
        }

        @Override
        protected void cleanup(Float64NDimArray instance) {
            FastTable.recycle(instance._sizes);
            instance._sizes = null;
            ArrayFactory.DOUBLES_FACTORY.recycle(instance._data);
            instance._data = null;
        }
    };

    private static Float64NDimArray newInstance(int numElements) {
        Float64NDimArray o = FACTORY.object();
        o._numElements = numElements;
        o._sizes = FastTable.newInstance();
        o._data = ArrayFactory.DOUBLES_FACTORY.array(numElements);
        Arrays.fill(o._data, 0, numElements, 0);
        return o;
    }

    private static Float64NDimArray copyOf(Float64NDimArray original) {
        Float64NDimArray o = Float64NDimArray.newInstance(original._numElements);
        o._sizes.addAll(original._sizes);
        System.arraycopy(original._data, 0, o._data, 0, original._numElements);
        return o;
    }

    private Float64NDimArray() {
    }

    /**
     * Tests the methods in this class.
     *
     * @param args Command-line arguments (ignored)
     */
    public static void main(String args[]) {
        System.out.println("Testing Float64NDimArray:");
        FastTable<Integer64> pos = FastTable.newInstance();
        pos.setSize(2);

        double[][] arr2D = {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
        Float64NDimArray nDArr = Float64NDimArray.valueOf(arr2D);
        System.out.println("nDArr = " + nDArr);
        pos.set(0, Integer64.valueOf(1));
        pos.set(1, Integer64.valueOf(1));
        System.out.println("  nDArr[1][1] = nDarr.get(pos) = " + nDArr.get(pos));
        System.out.println("nDArr.set(pos, 2)");
        nDArr.set(pos, 2);
        System.out.println("  nDArr[1][1] = nDarr.get(pos) = " + nDArr.get(pos));
        System.out.println("nDArr = " + nDArr);
        System.out.println("nDArr.vectorization() = " + nDArr.vectorization());

        Float64NDimArray nDArr2 = nDArr.copy();
        System.out.println("nDArr2 = nDArr.copy()");
        System.out.println("nDArr2 = " + nDArr2);
        System.out.println("nDArr2 = nDArr2.setValue(MathLib.PI)");
        nDArr2.setAll(MathLib.PI);
        System.out.println("nDArr2 = " + nDArr2);

        //  Write out XML data.
        try {
            System.out.println();
            javolution.xml.XMLObjectWriter writer = javolution.xml.XMLObjectWriter.newInstance(System.out);
            writer.setIndentation("    ");
            writer.write(nDArr, "Float64NDimArray", Float64NDimArray.class);
            writer.flush();

        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
