/**
*   Please feel free to use any fragment of the code in this file that you need
*   in your own work. As far as I am concerned, it's in the public domain. No
*   permission is necessary or required. Credit is always appreciated if you
*   use a large chunk or base a significant product on one of my examples,
*   but that's not required either.
*
*   This code is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*      --- Joseph A. Huwaldt
**/
package jahuwaldt.swing;


import javax.swing.*;
import java.util.List;


/**
*  A list model that models the case where the contents of a list (the data) can be
*  changed dynamically (such as when a user clicks on an item in another list).
*  To use this class, create a sub-class that simply defines the "getElement" method
*  of AbstractListModel.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author  Joseph A. Huwaldt   Date:  March 9, 2003
*  @version March 9, 2003
**/
public abstract class DynamicListModel extends AbstractListModel {

	//	The list of data objects to be displayed in this display list.
	private List data = null;
	
	//	The size of the list the last time "getSize()" was called.
	private int oldSize = 0;
	
	/**
	*  Default constructor.
	**/
	protected DynamicListModel() {}
	
	/**
	*  Constructor that takes a data list as an argument.  The supplied
	*  list becomes the data that this model uses as if it were passed
	*  to "setData()".
	**/
	protected DynamicListModel(List data) {
		this.data = data;
	}

	/**
	*  Returns the number of elements in the data list associated with this
	*  display list.  If there is no data associated with this display list
	*  then 0 is returned.
	**/
	public int getSize() {
		int size = 0;
		if (data != null)
			size = data.size();
		oldSize = size;
		return size;
	}
	
	/**
	*  Method called to change which data list is being displayed in this display list.
	**/
	public void setData(List data) {

		fireIntervalRemoved(this, 0, oldSize);
		this.data = data;
		
		if (data != null) {
			int idx2 = data.size() - 1;
			if (idx2 < 0)   idx2 = 0;
			fireIntervalAdded(this, 0, idx2);
		}
	}

	/**
	*  Removes the specified element from the display
	*  list.  This should only be called just before
	*  an element is removed from the data list.
	**/
	public void removeElement(int idx) {
		fireIntervalRemoved(this, idx, idx);
	}
	
	/**
	*  Returns a reference to the data list that this display list is referring to.
	**/
	public List getData() {
		return data;
	}
	
	/**
	*  This method is called to update the list after a new
	*  element has been added.
	**/
	public void elementAdded() {
		int pos = data.size() - 1;
		fireIntervalAdded(this, pos, pos);
	}

}
