/*
 *  xtlefn_sweepLE_xrle_b - Method for calculating X coord of tip chord LE as a function of
 *       sweepLE, xrle, & b.
 *
 *  Copyright (C) 2002 - 2015 by Joseph A. Huwaldt
 *  All rights reserved.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.js.ParamTools.AbstractParamMethod;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Length;
import javax.measure.unit.SI;

/**
 * Calculates X location of leading edge of tip chord as a function of leading
 * edge sweep, root chord LE X coord and true span.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt Date: May 23, 2002
 * @version June 19, 2015
 */
public class xtlefn_sweepLE_xrle_b extends AbstractParamMethod<Length> {

    /**
     * Constructor for this method.
     *
     * @param sweepLE Label for the "leading edge sweep" parameter.
     * @param xrle Label for the "X coord of root chord leading edge" parameter.
     * @param b Label for the "wing true span" parameter.
     */
    public xtlefn_sweepLE_xrle_b(String sweepLE, String xrle, String b) {
        _inputLabels = new String[3];
        _inputLabels[0] = sweepLE;
        _inputLabels[1] = xrle;
        _inputLabels[2] = b;
    }

    /**
     * Calculates X location of leading edge of tip chord as a function of
     * leading edge sweep, root chord LE X coord and true span. Method: 
     *      xtle = xrle + b*tan(sweepLE)
     *
     * @param inputs Array of input parameters. [0=sweep of wing LE (sweepLE),
     *      1=X location of root chord LE (xrle), 2=wing span (b)].
     * @return X coordinate of the leading edge of the root chord in length units.
     */
    @Override
    public Parameter<Length> calc(Parameter[] inputs) {
        double sweepLE = inputs[0].doubleValue(SI.RADIAN);
        double xrle = inputs[1].doubleValue(SI.METER);
        double b = inputs[2].doubleValue(SI.METER);
        double xtle = xrle + b * Math.tan(sweepLE);
        return Parameter.valueOf(xtle, SI.METER).to(inputs[1].getUnit());
    }

}
