/*
 *  yrlefn_b_dihed_ytle - Method for calculating Y coord of root chord LE as a function of
 *       b, dihed and ytle.
 *
 *  Copyright (C) 2002 - 2015 by Joseph A. Huwaldt
 *  All rights reserved.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.js.ParamTools.AbstractParamMethod;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Length;
import javax.measure.unit.SI;

/**
 * Calculates Y location of leading edge of root chord as a function of wing
 * true span, wing dihedral and Y location of tip chord LE.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt Date: May 23, 2002
 * @version June 19, 2015
 */
public class yrlefn_b_dihed_ytle extends AbstractParamMethod<Length> {

    /**
     * Constructor for this method.
     *
     * @param b Label for the "wing true span" parameter.
     * @param dihed Label for the "wing dihedral" parameter.
     * @param ytle Label for the "Y coord of tip chord leading edge" parameter.
     */
    public yrlefn_b_dihed_ytle(String b, String dihed, String ytle) {
        _inputLabels = new String[3];
        _inputLabels[0] = b;
        _inputLabels[1] = dihed;
        _inputLabels[2] = ytle;
    }

    /**
     * Calculates Y location of leading edge of root chord as a function of wing
     * true span, wing dihedral, and Y location of tip chord LE. Method:
     *      yrle = ytle - b*cos(dihed)
     *
     * @param inputs Array of input parameters. [0=wing span (b), 1=wing
     *  dihedral (dihed) 2=Y location of tip chord LE (ytle)].
     * @return Y coordinate of the leading edge of the root chord in length units.
     */
    @Override
    public Parameter<Length> calc(Parameter[] inputs) {
        double b = inputs[0].doubleValue(SI.METER);
        double dihed = inputs[1].doubleValue(SI.RADIAN);
        double ytle = inputs[2].doubleValue(SI.METER);
        double yrle = ytle - b * Math.cos(dihed);
        return Parameter.valueOf(yrle, SI.METER).to(inputs[0].getUnit());
    }
    
}
