/*
 *  ztlefn_dihed_b_zrle - Method for calculating Z location of tip chord LE as a function
 *               of dihedral, wing true span, and zrle.
 *
 *  Copyright (C) 2002 - 2015 by Joseph A. Huwaldt
 *  All rights reserved.
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/lgpl.html
 */

import jahuwaldt.js.ParamTools.AbstractParamMethod;
import jahuwaldt.js.param.Parameter;
import javax.measure.quantity.Length;
import javax.measure.unit.SI;

/**
 * Calculates the Z location of the tip chord LE as a function wing dihedral,
 * wing true span (not projected), and the Z location of the root chord LE.
 * 
 * <p> Modified by: Joseph A. Huwaldt </p>
 * 
 * @author Joseph A. Huwaldt Date: May 23, 2002
 * @version June 19, 2015
 */
public class ztlefn_dihed_b_zrle extends AbstractParamMethod<Length> {

    /**
     * Constructor for this method.
     *
     * @param dihed Label for the "wing dihedral" parameter.
     * @param b Label for the "wing true span" parameter.
     * @param zrle Label for the "Z coord of root chord leading edge" parameter.
     */
    public ztlefn_dihed_b_zrle(String dihed, String b, String zrle) {
        _inputLabels = new String[3];
        _inputLabels[0] = dihed;
        _inputLabels[1] = b;
        _inputLabels[2] = zrle;
    }

    /**
     * Calculates the Z location of the tip chord LE as a function of wing
     * dihedral, wing true span, and the Z location of the root chord LE.
     * Method: ztle = zrle + b*sin(dihed)
     *
     * @param inputs Array of input parameters. [0=wing dihedral (dihed), 1=wing
     * span (b), 2=Z location of root chord LE (zrle),].
     * @return Z location of the LE of the tip chord in length units (meters).
     */
    @Override
    public Parameter<Length> calc(Parameter[] inputs) {
        double dihed = inputs[0].doubleValue(SI.RADIAN);
        double b = inputs[1].doubleValue(SI.METER);
        double zrle = inputs[2].doubleValue(SI.METER);
        double ztle = zrle + b * Math.sin(dihed);
        return Parameter.valueOf(ztle, SI.METER).to(inputs[1].getUnit());
    }

}
